import os
import platform
import shutil
import stat
import sys
import unittest

import mock
import six
from parameterized.parameterized import parameterized
import pytest

from conans.client import tools
from conans.client.build.cmake import CMake
from conans.client.build.cmake_flags import cmake_in_local_cache_var_name
from conans.client.conf import get_default_settings_yml
from conans.client.tools import cross_building
from conans.client.tools.oss import cpu_count
from conans.errors import ConanException
from conans.model.build_info import CppInfo, DepsCppInfo
from conans.model.ref import ConanFileReference
from conans.model.settings import Settings
from conans.test.utils.mocks import MockSettings, ConanFileMock
from conans.test.utils.test_files import temp_folder
from conans.util.files import load, save
from conans.model.conf import ConfDefinition


def _format_path_as_cmake(pathstr):
    if platform.system() == "Windows":
        drive, path = os.path.splitdrive(pathstr)
        return drive.upper() + path.replace(os.path.sep, "/")
    return pathstr


class CMakeTest(unittest.TestCase):

    def setUp(self):
        self.tempdir = temp_folder(path_with_spaces=False)
        self.tempdir2 = temp_folder(path_with_spaces=False)

    def tearDown(self):
        shutil.rmtree(self.tempdir)
        shutil.rmtree(self.tempdir2)

    def test_config_patch(self):

        conanfile = ConanFileMock()
        conanfile.name = "MyPkg"
        conanfile.settings = Settings()
        conanfile.folders.set_base_source(os.path.join(self.tempdir, "src"))
        conanfile.folders.set_base_build(os.path.join(self.tempdir, "build"))
        conanfile.folders.set_base_package(os.path.join(self.tempdir, "pkg"))
        conanfile.deps_cpp_info = DepsCppInfo()

        msg = "FOLDER: " + _format_path_as_cmake(conanfile.package_folder)
        for folder in (conanfile.build_folder, conanfile.package_folder):
            save(os.path.join(folder, "file1.cmake"), "Nothing")
            save(os.path.join(folder, "file2"), msg)
            save(os.path.join(folder, "file3.txt"), msg)
            save(os.path.join(folder, "file3.cmake"), msg)
            save(os.path.join(folder, "sub", "file3.cmake"), msg)

        cmake = CMake(conanfile, generator="Unix Makefiles")
        cmake.patch_config_paths()
        for folder in (conanfile.build_folder, conanfile.package_folder):
            self.assertEqual("Nothing", load(os.path.join(folder, "file1.cmake")))
            self.assertEqual(msg, load(os.path.join(folder, "file2")))
            self.assertEqual(msg, load(os.path.join(folder, "file3.txt")))
            self.assertEqual("FOLDER: ${CONAN_MYPKG_ROOT}",
                             load(os.path.join(folder, "file3.cmake")))
            self.assertEqual("FOLDER: ${CONAN_MYPKG_ROOT}",
                             load(os.path.join(folder, "sub", "file3.cmake")))

    def test_config_patch_deps(self):
        conanfile = ConanFileMock()
        conanfile.name = "MyPkg"
        conanfile.settings = Settings()
        conanfile.folders.set_base_source(os.path.join(self.tempdir, "src"))
        conanfile.folders.set_base_build(os.path.join(self.tempdir, "build"))
        conanfile.folders.set_base_package(os.path.join(self.tempdir, "pkg"))
        conanfile.deps_cpp_info = DepsCppInfo()

        ref = ConanFileReference.loads("MyPkg1/0.1@user/channel")
        cpp_info = CppInfo(ref.name, self.tempdir2)
        conanfile.deps_cpp_info.add(ref.name, cpp_info)
        self.tempdir = temp_folder(path_with_spaces=False)

        self.assertEqual(list(conanfile.deps_cpp_info.deps), ['MyPkg1'])
        self.assertEqual(conanfile.deps_cpp_info['MyPkg1'].rootpath,
                         self.tempdir2)

        msg = "FOLDER: " + _format_path_as_cmake(self.tempdir2)
        for folder in (conanfile.build_folder, conanfile.package_folder):
            save(os.path.join(folder, "file1.cmake"), "Nothing")
            save(os.path.join(folder, "file2"), msg)
            save(os.path.join(folder, "file3.txt"), msg)
            save(os.path.join(folder, "file3.cmake"), msg)
            save(os.path.join(folder, "sub", "file3.cmake"), msg)

        cmake = CMake(conanfile, generator="Unix Makefiles")
        cmake.patch_config_paths()
        for folder in (conanfile.build_folder, conanfile.package_folder):
            self.assertEqual("Nothing", load(os.path.join(folder, "file1.cmake")))
            self.assertEqual(msg, load(os.path.join(folder, "file2")))
            self.assertEqual(msg, load(os.path.join(folder, "file3.txt")))
            self.assertEqual("FOLDER: ${CONAN_MYPKG1_ROOT}",
                             load(os.path.join(folder, "file3.cmake")))
            self.assertEqual("FOLDER: ${CONAN_MYPKG1_ROOT}",
                             load(os.path.join(folder, "sub", "file3.cmake")))

    def test_partial_build(self):
        conanfile = ConanFileMock()
        conanfile.settings = Settings()
        conanfile.should_configure = False
        conanfile.should_build = False
        conanfile.should_install = False
        conanfile.should_test = False
        cmake = CMake(conanfile, generator="Unix Makefiles")
        cmake.configure()
        self.assertIsNone(conanfile.command)
        cmake.build()
        self.assertIsNone(conanfile.command)
        cmake.install()
        self.assertIsNone(conanfile.command)
        conanfile.name = None
        cmake.patch_config_paths()
        cmake.test()
        self.assertIsNone(conanfile.command)

    def test_should_flags(self):
        conanfile = ConanFileMock()
        conanfile.settings = Settings()
        conanfile.should_configure = False
        conanfile.should_build = True
        conanfile.should_install = False
        conanfile.should_test = True
        conanfile.folders.set_base_package(temp_folder())
        cmake = CMake(conanfile, generator="Unix Makefiles")
        cmake.configure()
        self.assertIsNone(conanfile.command)
        cmake.build()
        self.assertIn("cmake --build %s" %
                      CMakeTest.scape(". -- -j%i" % cpu_count(output=conanfile.output)),
                      conanfile.command)
        cmake.install()
        self.assertNotIn("cmake --build %s" %
                         CMakeTest.scape(". --target install -- -j%i" %
                                         cpu_count(output=conanfile.output)), conanfile.command)
        cmake.test()
        self.assertIn("cmake --build %s" %
                      CMakeTest.scape(". --target test -- -j%i" %
                                      cpu_count(output=conanfile.output)),
                      conanfile.command)
        conanfile.should_build = False
        cmake.configure()
        self.assertNotIn("cd . && cmake", conanfile.command)
        cmake.build()
        self.assertNotIn("cmake --build %s" %
                         CMakeTest.scape(". -- -j%i" % cpu_count(output=conanfile.output)),
                         conanfile.command)
        cmake.install()
        self.assertNotIn("cmake --build %s" %
                         CMakeTest.scape(". --target install -- -j%i" %
                                         cpu_count(output=conanfile.output)), conanfile.command)
        cmake.test()
        self.assertIn("cmake --build %s" %
                      CMakeTest.scape(". --target test -- -j%i" %
                                      cpu_count(output=conanfile.output)),
                      conanfile.command)
        conanfile.should_install = True
        conanfile.should_test = False
        cmake.configure()
        self.assertNotIn("cd . && cmake", conanfile.command)
        cmake.build()
        self.assertNotIn("cmake --build %s" %
                         CMakeTest.scape(". -- -j%i" % cpu_count(output=conanfile.output)),
                         conanfile.command)
        cmake.install()
        self.assertIn("cmake --build %s" %
                      CMakeTest.scape(". --target install -- -j%i" %
                                      cpu_count(output=conanfile.output)), conanfile.command)
        cmake.test()
        self.assertNotIn("cmake --build %s" %
                         CMakeTest.scape(". --target test -- -j%i" %
                                         cpu_count(output=conanfile.output)), conanfile.command)

    def test_conan_run_tests(self):
        conanfile = ConanFileMock()
        conanfile.settings = Settings()
        conanfile.should_test = True
        cmake = CMake(conanfile, generator="Unix Makefiles")
        with tools.environment_append({"CONAN_RUN_TESTS": "0"}):
            cmake.test()
            self.assertIsNone(conanfile.command)

    def test_cmake_generator(self):
        conanfile = ConanFileMock()
        conanfile.settings = Settings()
        with tools.environment_append({"CONAN_CMAKE_GENERATOR": "My CMake Generator"}):
            cmake = CMake(conanfile)
            self.assertIn('-G "My CMake Generator"', cmake.command_line)

    def test_cmake_generator_intel(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "intel"
        settings.compiler.version = "19"
        settings.compiler.base = "Visual Studio"
        settings.compiler.base.version = "15"
        settings.arch = "x86_64"

        conanfile = ConanFileMock()
        conanfile.settings = settings

        cmake = CMake(conanfile)
        self.assertIn('-G "Visual Studio 15 2017 Win64"', cmake.command_line)
        self.assertIn('-T "Intel C++ Compiler 19.0', cmake.command_line)

    def test_cmake_custom_generator_intel(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "intel"
        settings.compiler.version = "19"
        settings.compiler.base = "Visual Studio"
        settings.compiler.base.version = "15"
        settings.arch = "x86_64"

        conanfile = ConanFileMock()
        conanfile.settings = settings

        with tools.environment_append({"CONAN_CMAKE_GENERATOR": "My CMake Generator"}):
            cmake = CMake(conanfile)
            self.assertIn('-G "My CMake Generator"', cmake.command_line)
            self.assertNotIn('-G "Visual Studio 15 2017" -A "x64"', cmake.command_line)
            self.assertNotIn('-T "Intel C++ Compiler 19.0', cmake.command_line)

    @parameterized.expand([("SunOS", "sparcv9", "sparc"),
                           ("AIX", "ppc64", "ppc32"),
                           ("SunOS", "x86_64", "x86")])
    def test_cmake_not_cross_compile(self, os_build, arch_build, arch):
        # https://github.com/conan-io/conan/issues/8052
        settings = Settings.loads(get_default_settings_yml())
        settings.os = os_build
        settings.os_build = os_build
        settings.compiler = "gcc"
        settings.compiler.version = "9.2"
        settings.compiler.libcxx = "libstdc++"
        settings.arch = arch
        settings.arch_build = arch_build

        conanfile = ConanFileMock()
        conanfile.settings = settings

        cmake = CMake(conanfile)
        self.assertNotIn('CMAKE_SYSTEM_NAME', cmake.command_line)
        self.assertIn('-G "Unix Makefiles"', cmake.command_line)

    def test_cmake_generator_platform(self):
        conanfile = ConanFileMock()
        conanfile.settings = Settings()

        with tools.environment_append({"CONAN_CMAKE_GENERATOR": "Green Hills MULTI",
                                       "CONAN_CMAKE_GENERATOR_PLATFORM": "My CMake Platform"}):
            cmake = CMake(conanfile)
            self.assertIn('-G "Green Hills MULTI" -A "My CMake Platform"', cmake.command_line)

    def test_cmake_generator_platform_override(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "15"
        settings.arch = "x86"

        conanfile = ConanFileMock()
        conanfile.settings = settings

        with tools.environment_append({"CONAN_CMAKE_GENERATOR_PLATFORM": "Win64"}):
            cmake = CMake(conanfile)
            self.assertIn('-G "Visual Studio 15 2017" -A "Win64"', cmake.command_line)

    def test_cmake_generator_argument(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "15"
        settings.compiler.toolset = "v141"
        settings.arch = "x86_64"

        conanfile = ConanFileMock()
        conanfile.settings = settings

        cmake = CMake(conanfile, generator="Visual Studio 16 2019")
        self.assertIn('-G "Visual Studio 16 2019" -A "x64"', cmake.command_line)
        cmake.build()
        self.assertIn("/verbosity:minimal", conanfile.command)
        cmake = CMake(conanfile, generator="Visual Studio 9 2008")
        self.assertIn('-G "Visual Studio 9 2008 Win64"', cmake.command_line)
        cmake.build()
        self.assertNotIn("verbosity", conanfile.command)

    def test_cmake_generator_platform_gcc(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Linux"
        settings.os_build = "Linux"
        settings.compiler = "gcc"
        settings.compiler.version = "8"
        settings.compiler.libcxx = "libstdc++"
        settings.arch = "x86"

        conanfile = ConanFileMock()
        conanfile.settings = settings

        cmake = CMake(conanfile)
        self.assertIn('-G "Unix Makefiles"', cmake.command_line)
        self.assertNotIn('-A', cmake.command_line)

    @parameterized.expand([('x86', 'Visual Studio 15 2017'),
                           ('x86_64', 'Visual Studio 15 2017 Win64'),
                           ('armv7', 'Visual Studio 15 2017 ARM')])
    def test_cmake_generator_platform_vs2017(self, arch, generator):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "15"
        settings.arch = arch

        conanfile = ConanFileMock()
        conanfile.settings = settings

        cmake = CMake(conanfile)
        self.assertIn('-G "%s"' % generator, cmake.command_line)
        self.assertNotIn('-A', cmake.command_line)

    @parameterized.expand([('x86', 'Win32'),
                           ('x86_64', 'x64'),
                           ('armv7', 'ARM'),
                           ('armv8', 'ARM64')])
    def test_cmake_generator_platform_vs2019(self, arch, pf):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "16"
        settings.arch = arch

        conanfile = ConanFileMock()
        conanfile.settings = settings

        cmake = CMake(conanfile)
        self.assertIn('-G "Visual Studio 16 2019" -A "%s"' % pf, cmake.command_line)

    def test_cmake_generator_platform_vs2019_with_ninja(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "16"
        settings.arch = "x86_64"

        conanfile = ConanFileMock()
        conanfile.settings = settings

        cmake = CMake(conanfile, generator="Ninja")
        self.assertIn('-G "Ninja"', cmake.command_line)
        self.assertNotIn("-A", cmake.command_line)

    @parameterized.expand([('arm',),
                           ('ppc',),
                           ('86',)])
    def test_cmake_generator_platform_other(self, arch):
        conanfile = ConanFileMock()
        conanfile.settings = Settings()

        with tools.environment_append({"CONAN_CMAKE_GENERATOR": "Green Hills MULTI",
                                       "CONAN_CMAKE_GENERATOR_PLATFORM": arch}):
            cmake = CMake(conanfile)
            self.assertIn('-G "Green Hills MULTI" -A "%s"' % arch, cmake.command_line)

    @parameterized.expand([('Ninja',),
                           ('NMake Makefiles',),
                           ('NMake Makefiles JOM',)
                           ])
    def test_generator_platform_with_unsupported_generator(self, generator):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "15"
        settings.arch = "x86"
        settings.compiler.toolset = "v140"

        conanfile = ConanFileMock()
        conanfile.settings = settings

        with self.assertRaises(ConanException):
            cmake = CMake(conanfile, generator=generator, generator_platform="x64")
            _ = cmake.command_line

    def test_cmake_fpic(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Linux"
        settings.compiler = "gcc"
        settings.compiler.version = "6.3"
        settings.arch = "x86"

        def assert_fpic(the_settings, input_shared, input_fpic, expected_option):
            options = []
            values = {}
            if input_shared is not None:
                options.append('"shared": [True, False]')
                values["shared"] = input_shared
            if input_fpic is not None:
                options.append('"fPIC": [True, False]')
                values["fPIC"] = input_fpic

            conanfile = ConanFileMock(options='{%s}' % ", ".join(options),
                                      options_values=values)
            conanfile.settings = the_settings
            cmake = CMake(conanfile)
            cmake.configure()
            if expected_option is not None:
                self.assertEqual(cmake.definitions["CONAN_CMAKE_POSITION_INDEPENDENT_CODE"],
                                 expected_option)
            else:
                self.assertNotIn("CONAN_CMAKE_POSITION_INDEPENDENT_CODE", cmake.definitions)

        # Test shared=False and fpic=False
        assert_fpic(settings, input_shared=False, input_fpic=False, expected_option="OFF")

        # Test shared=True and fpic=False
        assert_fpic(settings, input_shared=True, input_fpic=False, expected_option="ON")

        # Test shared=True and fpic=True
        assert_fpic(settings, input_shared=True, input_fpic=True, expected_option="ON")

        # Test shared not defined and fpic=True
        assert_fpic(settings, input_shared=None, input_fpic=True, expected_option="ON")

        # Test shared not defined and fpic not defined
        assert_fpic(settings, input_shared=None, input_fpic=None, expected_option=None)

        # Test shared True and fpic not defined
        assert_fpic(settings, input_shared=True, input_fpic=None, expected_option=None)

        # Test nothing in Windows
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "15"
        settings.arch = "x86_64"

        assert_fpic(settings, input_shared=True, input_fpic=True, expected_option=None)

    def test_cmake_make_program(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Linux"
        settings.compiler = "gcc"
        settings.compiler.version = "6.3"
        settings.arch = "x86"
        settings.build_type = "Release"
        conanfile = ConanFileMock()
        conanfile.settings = settings
        conanfile.folders.set_base_source(os.path.join(self.tempdir, "my_cache_source_folder"))
        conanfile.folders.set_base_build(os.path.join(self.tempdir, "my_cache_build_folder"))

        # Existing make
        make_path = os.path.join(self.tempdir, "make")
        save(make_path, "")
        st = os.stat(make_path)
        os.chmod(make_path, st.st_mode | stat.S_IEXEC)
        with tools.environment_append({"CONAN_MAKE_PROGRAM": make_path}):
            cmake = CMake(conanfile)
            self.assertEqual(cmake.definitions["CMAKE_MAKE_PROGRAM"], make_path)

        # Not existing make
        with tools.environment_append({"CONAN_MAKE_PROGRAM": "fake_path/make"}):
            cmake = CMake(conanfile)
            self.assertNotIn("CMAKE_MAKE_PROGRAM", cmake.definitions)
            self.assertIn("The specified make program 'fake_path/make' cannot be found",
                          conanfile.output)

    def test_folders(self):
        def quote_var(var):
            return "'%s'" % var if platform.system() != "Windows" else var

        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Linux"
        settings.compiler = "gcc"
        settings.compiler.version = "6.3"
        settings.arch = "x86"
        settings.build_type = "Release"

        conanfile = ConanFileMock()
        conanfile.settings = settings
        conanfile.folders.set_base_source(os.path.join(self.tempdir, "my_cache_source_folder"))
        conanfile.folders.set_base_build(os.path.join(self.tempdir, "my_cache_build_folder"))
        with tools.chdir(self.tempdir):
            linux_stuff = ""
            if platform.system() != "Linux":
                linux_stuff = '-DCMAKE_SYSTEM_NAME="Linux" -DCMAKE_SYSROOT="/path/to/sysroot" '
            generator = "MinGW Makefiles" if platform.system() == "Windows" else "Unix Makefiles"

            flags = '{} -DCONAN_COMPILER="gcc" ' \
                    '-DCONAN_COMPILER_VERSION="6.3" ' \
                    '-DCONAN_CXX_FLAGS="-m32" -DCONAN_SHARED_LINKER_FLAGS="-m32" ' \
                    '-DCONAN_C_FLAGS="-m32" -DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" ' \
                    '-DCONAN_EXPORTED="1"'
            flags_in_local_cache = flags.format('-D' + cmake_in_local_cache_var_name + '="ON"')
            flags_no_local_cache = flags.format('-D' + cmake_in_local_cache_var_name + '="OFF"')

            base_cmd = 'cmake -G "{generator}" -DCMAKE_BUILD_TYPE="Release" {linux_stuff}' \
                       '{{flags}} -Wno-dev'
            base_cmd = base_cmd.format(generator=generator, linux_stuff=linux_stuff)
            full_cmd = "cd {build_expected} && {base_cmd} {source_expected}"

            build_expected = quote_var("build")
            source_expected = quote_var("../subdir")

            cmake = CMake(conanfile)
            cmake.configure(source_dir="../subdir", build_dir="build")
            self.assertEqual(conanfile.command,
                             full_cmd.format(build_expected=build_expected,
                                             source_expected=source_expected,
                                             base_cmd=base_cmd.format(flags=flags_no_local_cache)))

            cmake = CMake(conanfile)
            cmake.configure(build_dir="build")
            build_expected = quote_var("build")
            source_expected = quote_var(os.path.join(self.tempdir, "my_cache_source_folder"))
            self.assertEqual(conanfile.command,
                             full_cmd.format(build_expected=build_expected,
                                             source_expected=source_expected,
                                             base_cmd=base_cmd.format(flags=flags_no_local_cache)))

            cmake = CMake(conanfile)
            cmake.configure()
            build_expected = quote_var(os.path.join(self.tempdir, "my_cache_build_folder"))
            source_expected = quote_var(os.path.join(self.tempdir, "my_cache_source_folder"))
            self.assertEqual(conanfile.command,
                             full_cmd.format(build_expected=build_expected,
                                             source_expected=source_expected,
                                             base_cmd=base_cmd.format(flags=flags_no_local_cache)))

            cmake = CMake(conanfile)
            cmake.configure(source_folder="source", build_folder="build")
            build_expected = quote_var(os.path.join(os.path.join(self.tempdir,
                                                                 "my_cache_build_folder", "build")))
            source_expected = quote_var(os.path.join(os.path.join(self.tempdir,
                                                                  "my_cache_source_folder",
                                                                  "source")))
            self.assertEqual(conanfile.command,
                             full_cmd.format(build_expected=build_expected,
                                             source_expected=source_expected,
                                             base_cmd=base_cmd.format(flags=flags_no_local_cache)))

            conanfile.in_local_cache = True
            cmake = CMake(conanfile)
            cmake.configure(source_folder="source", build_folder="build",
                            cache_build_folder="rel_only_cache")
            build_expected = quote_var(os.path.join(self.tempdir, "my_cache_build_folder",
                                                    "rel_only_cache"))
            source_expected = quote_var(os.path.join(self.tempdir, "my_cache_source_folder",
                                                     "source"))
            self.assertEqual(conanfile.command,
                             full_cmd.format(build_expected=build_expected,
                                             source_expected=source_expected,
                                             base_cmd=base_cmd.format(flags=flags_in_local_cache)))

            conanfile.in_local_cache = False
            cmake = CMake(conanfile)
            cmake.configure(source_folder="source", build_folder="build",
                            cache_build_folder="rel_only_cache")
            build_expected = quote_var(os.path.join(self.tempdir, "my_cache_build_folder", "build"))
            source_expected = quote_var(os.path.join(self.tempdir, "my_cache_source_folder",
                                                     "source"))
            self.assertEqual(conanfile.command,
                             full_cmd.format(build_expected=build_expected,
                                             source_expected=source_expected,
                                             base_cmd=base_cmd.format(flags=flags_no_local_cache)))

            conanfile.in_local_cache = True
            cmake = CMake(conanfile)
            cmake.configure(build_dir="build", cache_build_folder="rel_only_cache")
            build_expected = quote_var(os.path.join(self.tempdir, "my_cache_build_folder",
                                                    "rel_only_cache"))
            source_expected = quote_var(os.path.join(self.tempdir, "my_cache_source_folder"))
            self.assertEqual(conanfile.command, full_cmd.format(build_expected=build_expected,
                                                                source_expected=source_expected,
                                                                base_cmd=base_cmd.format(
                                                                        flags=flags_in_local_cache)))

            # Raise mixing
            with six.assertRaisesRegex(self, ConanException, "Use 'build_folder'/'source_folder'"):
                cmake = CMake(conanfile)
                cmake.configure(source_folder="source", build_dir="build")

    def test_build_type_force(self):
        # 1: No multi-config generator
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Linux"
        settings.compiler = "gcc"
        settings.compiler.version = "6.3"
        settings.arch = "x86"
        settings.build_type = "Release"
        conanfile = ConanFileMock()
        conanfile.settings = settings
        # 2: build_type from settings
        cmake = CMake(conanfile)
        self.assertNotIn('WARN: Forced CMake build type ', conanfile.output)
        self.assertEqual(cmake.build_type, "Release")

        # 2: build_type from attribute
        cmake.build_type = "Debug"
        expected_output = "WARN: Forced CMake build type ('Debug') different from the settings " \
                          "build type ('Release')"
        self.assertIn(expected_output, conanfile.output)
        self.assertEqual(cmake.build_type, "Debug")
        self.assertIn('-DCMAKE_BUILD_TYPE="Debug"', cmake.command_line)

        # 2: build_type from constructor
        cmake = CMake(conanfile, build_type="Debug")
        expected_output = "WARN: Forced CMake build type ('Debug') different from the settings " \
                          "build type ('Release')"
        self.assertIn(expected_output, conanfile.output)
        self.assertEqual(cmake.build_type, "Debug")
        self.assertIn('-DCMAKE_BUILD_TYPE="Debug"', cmake.command_line)

        # 1: Multi-config generator
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "15"
        settings.arch = "x86"
        settings.build_type = "Release"
        conanfile = ConanFileMock()
        conanfile.settings = settings
        # 2: build_type from settings
        cmake = CMake(conanfile)
        self.assertNotIn('-DCMAKE_BUILD_TYPE="Release"', cmake.command_line)
        self.assertIn("--config Release", cmake.build_config)

        # 2: build_type from attribute
        cmake.build_type = "Debug"
        self.assertIn(expected_output, conanfile.output)
        self.assertEqual(cmake.build_type, "Debug")
        self.assertNotIn('-DCMAKE_BUILD_TYPE="Debug"', cmake.command_line)
        self.assertIn("--config Debug", cmake.build_config)

        # 2: build_type from constructor
        cmake = CMake(conanfile, build_type="Debug")
        self.assertIn(expected_output, conanfile.output)
        self.assertEqual(cmake.build_type, "Debug")
        self.assertNotIn('-DCMAKE_BUILD_TYPE="Debug"', cmake.command_line)
        self.assertIn("--config Debug", cmake.build_config)

    def test_loads_default(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "12"
        settings.arch = "x86"
        conanfile = ConanFileMock()
        conanfile.settings = settings

        def check(text, build_config, generator=None, set_cmake_flags=False):
            the_os = str(settings.os)
            os_ver = str(settings.os.version) if settings.get_safe('os.version') else None
            for cmake_system_name in (True, False):
                cross_ver = ("-DCMAKE_SYSTEM_VERSION=\"%s\" " % os_ver) if os_ver else ""
                # FIXME: This test is complicated to maintain and see the logic, lets simplify it
                cross = ""
                skip_x64_x86 = the_os in ['Windows', 'Linux']
                if cmake_system_name and cross_building(conanfile, skip_x64_x86=skip_x64_x86):
                    cross = ("-DCMAKE_SYSTEM_NAME=\"%s\" %s-DCMAKE_SYSROOT=\"/path/to/sysroot\" "
                             % ({"Macos": "Darwin"}.get(the_os, the_os), cross_ver))
                cmake = CMake(conanfile, generator=generator, cmake_system_name=cmake_system_name,
                              set_cmake_flags=set_cmake_flags)
                new_text = text.replace("-DCONAN_IN_LOCAL_CACHE", "%s-DCONAN_IN_LOCAL_CACHE" % cross)
                if "Visual Studio" in text:
                    cores = ('-DCONAN_CXX_FLAGS="/MP{0}" '
                             '-DCONAN_C_FLAGS="/MP{0}" '.format(tools.cpu_count(conanfile.output)))
                    new_text = new_text.replace('-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON"',
                                                '%s-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON"' % cores)
                self.assertEqual(new_text, cmake.command_line)
                self.assertEqual(build_config, cmake.build_config)

        check('-G "Visual Studio 12 2013" -DCONAN_IN_LOCAL_CACHE="OFF" '
              '-DCONAN_COMPILER="Visual Studio" -DCONAN_COMPILER_VERSION="12" '
              '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" -Wno-dev',
              "")

        check('-G "Custom Generator" -DCONAN_IN_LOCAL_CACHE="OFF" '
              '-DCONAN_COMPILER="Visual Studio" -DCONAN_COMPILER_VERSION="12" '
              '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" -Wno-dev',
              '', generator="Custom Generator")

        check('-G "Custom Generator" -DCONAN_IN_LOCAL_CACHE="OFF" '
              '-DCONAN_COMPILER="Visual Studio" -DCONAN_COMPILER_VERSION="12" '
              '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" -Wno-dev',
              '', generator="Custom Generator", set_cmake_flags=True)

        settings.build_type = "Debug"
        check('-G "Visual Studio 12 2013" -DCONAN_IN_LOCAL_CACHE="OFF" '
              '-DCONAN_COMPILER="Visual Studio" -DCONAN_COMPILER_VERSION="12" '
              '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" -Wno-dev',
              '--config Debug')

        settings.arch = "x86_64"
        check('-G "Visual Studio 12 2013 Win64" -DCONAN_IN_LOCAL_CACHE="OFF" '
              '-DCONAN_COMPILER="Visual Studio" -DCONAN_COMPILER_VERSION="12" '
              '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" -Wno-dev',
              '--config Debug')

        settings.compiler = "gcc"
        settings.compiler.version = "4.8"
        cmakegen = "MinGW Makefiles" if platform.system() == "Windows" else "Unix Makefiles"
        check('-G "%s" -DCMAKE_BUILD_TYPE="Debug" -DCONAN_IN_LOCAL_CACHE="OFF" '
              '-DCONAN_COMPILER="gcc" -DCONAN_COMPILER_VERSION="4.8" -DCONAN_CXX_FLAGS="-m64" '
              '-DCONAN_SHARED_LINKER_FLAGS="-m64" -DCONAN_C_FLAGS="-m64" '
              '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" '
              '-Wno-dev' % cmakegen, "")

        settings.os = "Linux"
        settings.arch = "x86"
        check('-G "%s" -DCMAKE_BUILD_TYPE="Debug" '
              '-DCONAN_IN_LOCAL_CACHE="OFF" -DCONAN_COMPILER="gcc" '
              '-DCONAN_COMPILER_VERSION="4.8" -DCONAN_CXX_FLAGS="-m32" '
              '-DCONAN_SHARED_LINKER_FLAGS="-m32" -DCONAN_C_FLAGS="-m32" '
              '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" -Wno-dev' % cmakegen,
              "")

        settings.arch = "armv7"
        check('-G "%s" -DCMAKE_BUILD_TYPE="Debug" '
              '-DCONAN_IN_LOCAL_CACHE="OFF" -DCONAN_COMPILER="gcc" '
              '-DCONAN_COMPILER_VERSION="4.8" '
              '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" -Wno-dev' % cmakegen,
              "")

        settings.arch = "x86_64"
        check('-G "%s" -DCMAKE_BUILD_TYPE="Debug" '
              '-DCONAN_IN_LOCAL_CACHE="OFF" -DCONAN_COMPILER="gcc" '
              '-DCONAN_COMPILER_VERSION="4.8" -DCONAN_CXX_FLAGS="-m64" '
              '-DCONAN_SHARED_LINKER_FLAGS="-m64" -DCONAN_C_FLAGS="-m64" '
              '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" -Wno-dev' % cmakegen,
              "")

        check('-G "%s" -DCMAKE_BUILD_TYPE="Debug" '
              '-DCONAN_IN_LOCAL_CACHE="OFF" -DCONAN_COMPILER="gcc" '
              '-DCONAN_COMPILER_VERSION="4.8" -DCONAN_CXX_FLAGS="-m64" '
              '-DCONAN_SHARED_LINKER_FLAGS="-m64" -DCONAN_C_FLAGS="-m64" '
              '-DCMAKE_CXX_FLAGS="-m64" -DCMAKE_SHARED_LINKER_FLAGS="-m64" -DCMAKE_C_FLAGS="-m64" '
              '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" '
              '-Wno-dev' % cmakegen,
              "", set_cmake_flags=True)

        settings.os = "FreeBSD"
        settings.compiler = "clang"
        settings.compiler.version = "3.8"
        settings.arch = "x86"
        check('-G "%s" -DCMAKE_BUILD_TYPE="Debug" '
              '-DCONAN_IN_LOCAL_CACHE="OFF" -DCONAN_COMPILER="clang" '
              '-DCONAN_COMPILER_VERSION="3.8" -DCONAN_CXX_FLAGS="-m32" '
              '-DCONAN_SHARED_LINKER_FLAGS="-m32" -DCONAN_C_FLAGS="-m32" '
              '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" -Wno-dev' % cmakegen,
              "")

        settings.arch = "x86_64"
        check('-G "%s" -DCMAKE_BUILD_TYPE="Debug" '
              '-DCONAN_IN_LOCAL_CACHE="OFF" -DCONAN_COMPILER="clang" '
              '-DCONAN_COMPILER_VERSION="3.8" -DCONAN_CXX_FLAGS="-m64" '
              '-DCONAN_SHARED_LINKER_FLAGS="-m64" -DCONAN_C_FLAGS="-m64" '
              '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" -Wno-dev' % cmakegen,
              "")

        settings.os = "SunOS"
        settings.compiler = "sun-cc"
        settings.compiler.version = "5.10"
        settings.arch = "x86"
        check('-G "%s" -DCMAKE_BUILD_TYPE="Debug" '
              '-DCONAN_IN_LOCAL_CACHE="OFF" -DCONAN_COMPILER="sun-cc" '
              '-DCONAN_COMPILER_VERSION="5.10" -DCONAN_CXX_FLAGS="-m32" '
              '-DCONAN_SHARED_LINKER_FLAGS="-m32" -DCONAN_C_FLAGS="-m32" '
              '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" -Wno-dev' % cmakegen,
              "")

        settings.arch = "x86_64"
        check('-G "%s" -DCMAKE_BUILD_TYPE="Debug" '
              '-DCONAN_IN_LOCAL_CACHE="OFF" -DCONAN_COMPILER="sun-cc" '
              '-DCONAN_COMPILER_VERSION="5.10" -DCONAN_CXX_FLAGS="-m64" '
              '-DCONAN_SHARED_LINKER_FLAGS="-m64" -DCONAN_C_FLAGS="-m64" '
              '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" -Wno-dev' % cmakegen,
              "")

        settings.arch = "sparc"

        check('-G "%s" -DCMAKE_BUILD_TYPE="Debug" -DCONAN_IN_LOCAL_CACHE="OFF" '
              '-DCONAN_COMPILER="sun-cc" '
              '-DCONAN_COMPILER_VERSION="5.10" -DCONAN_CXX_FLAGS="-m32" '
              '-DCONAN_SHARED_LINKER_FLAGS="-m32" -DCONAN_C_FLAGS="-m32" '
              '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" -Wno-dev' % cmakegen,
              "")

        settings.arch = "sparcv9"
        check('-G "%s" -DCMAKE_BUILD_TYPE="Debug" -DCONAN_IN_LOCAL_CACHE="OFF" '
              '-DCONAN_COMPILER="sun-cc" '
              '-DCONAN_COMPILER_VERSION="5.10" -DCONAN_CXX_FLAGS="-m64" '
              '-DCONAN_SHARED_LINKER_FLAGS="-m64" -DCONAN_C_FLAGS="-m64" '
              '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" -Wno-dev' % cmakegen,
              "")

        settings.compiler = "Visual Studio"
        settings.compiler.version = "12"
        settings.os = "WindowsStore"
        settings.os.version = "8.1"
        settings.build_type = "Debug"
        check('-G "Visual Studio 12 2013" -DCONAN_IN_LOCAL_CACHE="OFF" '
              '-DCONAN_COMPILER="Visual Studio" -DCONAN_COMPILER_VERSION="12" '
              '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" -Wno-dev',
              "--config Debug")

        settings.os.version = "10.0"
        check('-G "Visual Studio 12 2013" -DCONAN_IN_LOCAL_CACHE="OFF" '
              '-DCONAN_COMPILER="Visual Studio" -DCONAN_COMPILER_VERSION="12" '
              '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" -Wno-dev',
              "--config Debug")

        settings.compiler.version = "15"
        settings.arch = "armv8"
        check('-G "Visual Studio 15 2017" -A "ARM64" -DCONAN_IN_LOCAL_CACHE="OFF" '
              '-DCONAN_COMPILER="Visual Studio" -DCONAN_COMPILER_VERSION="15" '
              '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" -Wno-dev',
              "--config Debug")

        settings.arch = "x86_64"
        check('-G "Visual Studio 15 2017 Win64" -DCONAN_IN_LOCAL_CACHE="OFF" '
              '-DCONAN_COMPILER="Visual Studio" -DCONAN_COMPILER_VERSION="15" '
              '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" -Wno-dev',
              "--config Debug")

        settings.compiler = "Visual Studio"
        settings.compiler.version = "9"
        settings.os = "WindowsCE"
        settings.os.platform = "Your platform name (ARMv4I)"
        settings.os.version = "7.0"
        settings.build_type = "Debug"
        check('-G "Visual Studio 9 2008" '
              '-A "Your platform name (ARMv4I)" '
              '-DCONAN_IN_LOCAL_CACHE="OFF" '
              '-DCONAN_COMPILER="Visual Studio" -DCONAN_COMPILER_VERSION="9" '
              '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" -Wno-dev',
              "--config Debug")

    def test_deleted_os(self):
        partial_settings = """
os: [Linux]
arch: [x86_64]
compiler:
    gcc:
        version: ["4.9"]
build_type: [ Release]
"""
        settings = Settings.loads(partial_settings)
        settings.os = "Linux"
        settings.compiler = "gcc"
        settings.compiler.version = "4.9"
        settings.arch = "x86_64"
        conanfile = ConanFileMock()
        conanfile.settings = settings

        cmake = CMake(conanfile)
        generator = "Unix" if platform.system() != "Windows" else "MinGW"
        cross = ("-DCMAKE_SYSTEM_NAME=\"Linux\" -DCMAKE_SYSROOT=\"/path/to/sysroot\" "
                 if platform.system() != "Linux" else "")
        self.assertEqual('-G "%s Makefiles" %s-DCONAN_IN_LOCAL_CACHE="OFF" '
                         '-DCONAN_COMPILER="gcc" '
                         '-DCONAN_COMPILER_VERSION="4.9" -DCONAN_CXX_FLAGS="-m64" '
                         '-DCONAN_SHARED_LINKER_FLAGS="-m64" -DCONAN_C_FLAGS="-m64" '
                         '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" '
                         '-Wno-dev' % (generator, cross),
                         cmake.command_line)

    def test_sysroot(self):
        settings = Settings.loads(get_default_settings_yml())
        conanfile = ConanFileMock()
        conanfile.settings = settings
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "12"
        settings.arch = "x86"
        if platform.system() == "Windows":
            cmake = CMake(conanfile)
            self.assertNotIn("-DCMAKE_SYSROOT=", cmake.flags)

        # Now activate cross build and check sysroot and system processor
        with(tools.environment_append({"CONAN_CMAKE_SYSTEM_NAME": "Android",
                                       "CONAN_CMAKE_SYSTEM_PROCESSOR": "somevalue"})):
            cmake = CMake(conanfile)
            self.assertEqual(cmake.definitions["CMAKE_SYSROOT"], "/path/to/sysroot")
            self.assertEqual(cmake.definitions["CMAKE_SYSTEM_PROCESSOR"], "somevalue")

    def test_sysroot_envvar(self):
        settings = Settings.loads(get_default_settings_yml())
        conanfile = ConanFileMock()
        conanfile.settings = settings
        settings.os = "Linux"
        settings.os_build = "Windows"
        settings.compiler = "gcc"
        settings.compiler.version = "5"
        settings.arch_build = "x86_64"
        settings.arch = "armv7"

        # Now activate cross build and check sysroot and system processor
        with(tools.environment_append({"CONAN_CMAKE_SYSROOT": "/path/to/var/sysroot"})):
            cmake = CMake(conanfile)
            self.assertEqual(cmake.definitions["CMAKE_SYSROOT"], "/path/to/var/sysroot")

    def test_deprecated_behaviour(self):
        """"Remove when deprecate the old settings parameter to CMake and
        conanfile to configure/build/test"""
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        conanfile = ConanFileMock()
        conanfile.settings = settings
        with self.assertRaises(ConanException):
            CMake(settings)

    def test_cores_ancient_visual(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "9"
        settings.compiler.runtime = "MDd"
        settings.arch = "x86"
        settings.build_type = None

        conanfile = ConanFileMock()
        conanfile.settings = settings
        cmake = CMake(conanfile)

        cmake.build()
        self.assertNotIn("/m", conanfile.command)

        settings.compiler.version = "10"
        cmake = CMake(conanfile)

        cmake.build()
        self.assertIn("/m", conanfile.command)

    def test_convenient_functions(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Android"
        settings.os.api_level = 16
        settings.os_build = "Windows"  # Here we are declaring we are cross building
        settings.compiler = "gcc"
        settings.compiler.version = "5.4"
        settings.arch = "armv7"
        settings.build_type = None

        if platform.system() == 'Windows':
            dot_dir = "."
            tempdir = self.tempdir
        else:
            dot_dir = "'.'"
            tempdir = "'" + self.tempdir + "'"

        conanfile = ConanFileMock()
        conanfile.settings = settings
        cmake = CMake(conanfile)

        cross = '-DCMAKE_SYSTEM_NAME="Android"' \
                ' -DCMAKE_SYSTEM_VERSION="{0}"' \
                ' -DCMAKE_SYSROOT="/path/to/sysroot"' \
                ' -DCMAKE_ANDROID_ARCH_ABI="armeabi-v7a"' \
                ' -DANDROID_ABI="armeabi-v7a"' \
                ' -DANDROID_PLATFORM="android-{0}"' \
                ' -DANDROID_TOOLCHAIN="{1}"' \
                ' -DANDROID_STL="none"'.format(settings.os.api_level, settings.compiler)
        target_test = CMakeTest.scape('--target test')

        cmake.configure()

        self.assertEqual('cd {0} && cmake -G "MinGW Makefiles" '
                         '{1} -DCONAN_IN_LOCAL_CACHE="OFF"'
                         ' -DCONAN_COMPILER="{2}" -DCONAN_COMPILER_VERSION="{3}"'
                         ' -DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON"'
                         ' -DCONAN_EXPORTED="1"'
                         ' -Wno-dev {0}'.format(dot_dir, cross, settings.compiler,
                                                settings.compiler.version),
                         conanfile.command)

        cmake.build()
        self.assertEqual('cmake --build %s %s' %
                         (dot_dir, (CMakeTest.scape('-- -j%i' % cpu_count(conanfile.output)))),
                         conanfile.command)

        cmake.test()
        self.assertEqual('cmake --build %s %s %s' %
                         (dot_dir, target_test,
                          (CMakeTest.scape('-- -j%i' % cpu_count(output=conanfile.output)))),
                         conanfile.command)

        settings.build_type = "Debug"
        cmake = CMake(conanfile)
        cmake.build()
        self.assertEqual('cmake --build %s %s' %
                         (dot_dir, (CMakeTest.scape('-- -j%i' %
                                                    cpu_count(output=conanfile.output)))),
                         conanfile.command)

        cmake.test()
        self.assertEqual('cmake --build %s %s %s' %
                         (dot_dir, target_test,
                          (CMakeTest.scape('-- -j%i' % cpu_count(output=conanfile.output)))),
                         conanfile.command)

        cmake.configure(source_dir="/source", build_dir=self.tempdir,
                        args=['--foo "bar"'], defs={"SHARED": True})
        if sys.platform == 'win32':
            escaped_args = r'"--foo \"bar\"" -DSHARED="True" /source'
        else:
            escaped_args = "'--foo \"bar\"' -DSHARED=\"True\" '/source'"

        self.assertEqual('cd {0} && cmake -G "MinGW Makefiles" -DCMAKE_BUILD_TYPE="Debug" '
                         '{1} -DCONAN_IN_LOCAL_CACHE="OFF" '
                         '-DCONAN_COMPILER="{2}" -DCONAN_COMPILER_VERSION="{3}" '
                         '-DCMAKE_EXPORT_NO_PACKAGE_REGISTRY="ON" -DCONAN_EXPORTED="1" '
                         '-Wno-dev {4}'.format(tempdir, cross, settings.compiler,
                                               settings.compiler.version, escaped_args),
                         conanfile.command)

        cmake.build(args=["--bar 'foo'"], target="install")
        if platform.system() == 'Windows':
            escaped_args = '--target install "--bar \'foo\'"'
        else:
            escaped_args = r"'--target' 'install' '--bar '\''foo'\'''"
        self.assertEqual('cmake --build %s %s %s' %
                         (tempdir, escaped_args,
                          (CMakeTest.scape('-- -j%i' % cpu_count(output=conanfile.output)))),
                         conanfile.command)

        cmake.test(args=["--bar 'foo'"])
        if sys.platform == 'win32':
            escaped_args = '%s "--bar \'foo\'"' % target_test
        else:
            escaped_args = r"%s '--bar '\''foo'\'''" % target_test
        self.assertEqual('cmake --build %s %s %s' %
                         (tempdir, escaped_args,
                          (CMakeTest.scape('-- -j%i' % cpu_count(output=conanfile.output)))),
                         conanfile.command)

        settings.build_type = "Release"
        cmake = CMake(conanfile)
        cmake.build()
        self.assertEqual('cmake --build %s %s' %
                         (dot_dir, (CMakeTest.scape('-- -j%i' %
                                                    cpu_count(output=conanfile.output)))),
                         conanfile.command)

        cmake.test()
        self.assertEqual('cmake --build %s %s %s'
                         % (dot_dir, target_test,
                            (CMakeTest.scape('-- -j%i' % cpu_count(output=conanfile.output)))),
                         conanfile.command)

        cmake.build(build_dir=self.tempdir)
        self.assertEqual('cmake --build %s %s'
                         % (tempdir, (CMakeTest.scape('-- -j%i' %
                                                      cpu_count(output=conanfile.output)))),
                         conanfile.command)

        cmake.test(build_dir=self.tempdir)
        self.assertEqual('cmake --build %s %s %s' %
                         (tempdir, target_test,
                          (CMakeTest.scape('-- -j%i' % cpu_count(output=conanfile.output)))),
                         conanfile.command)

        settings.compiler = "gcc"
        settings.compiler.version = "5.4"
        cmake = CMake(conanfile)
        cmake.build()
        self.assertEqual('cmake --build %s' %
                         (CMakeTest.scape('. -- -j%i' % cpu_count(output=conanfile.output))),
                         conanfile.command)

        cmake.test()
        self.assertEqual('cmake --build %s' %
                         (CMakeTest.scape('. --target test -- -j%i' %
                                          cpu_count(output=conanfile.output))),
                         conanfile.command)

        cmake.build(args=['foo', '--', 'bar'])
        self.assertEqual('cmake --build %s' %
                         (CMakeTest.scape('. foo -- bar -j%i' %
                                          cpu_count(output=conanfile.output))),
                         conanfile.command)

        cmake.test(args=['foo', '--', 'bar'])
        self.assertEqual('cmake --build %s' %
                         (CMakeTest.scape('. --target test foo -- bar -j%i' %
                                          cpu_count(output=conanfile.output))),
                         conanfile.command)

        cmake = CMake(conanfile, parallel=False)
        cmake.build()
        self.assertEqual('cmake --build %s' % CMakeTest.scape('.'), conanfile.command)

        cmake.test()
        self.assertEqual('cmake --build %s' % CMakeTest.scape('. --target test'),
                         conanfile.command)

    def test_run_tests(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "14"
        settings.compiler.runtime = "MDd"
        settings.arch = "x86"
        settings.build_type = None

        conanfile = ConanFileMock()
        conanfile.settings = settings
        cmake = CMake(conanfile)
        cmake.test()
        self.assertIn('cmake --build '
                      '%s' % CMakeTest.scape('. --target RUN_TESTS -- /m:%i' %
                                             cpu_count(output=conanfile.output)),
                      conanfile.command)

        cmake.generator = "Ninja Makefiles"
        cmake.test()
        self.assertEqual('cmake --build '
                         '%s' % CMakeTest.scape('. --target test -- -j%i' %
                                                cpu_count(output=conanfile.output)),
                         conanfile.command)

        cmake.generator = "Ninja"
        cmake.test()
        self.assertEqual('cmake --build '
                         '%s' % CMakeTest.scape('. --target test -- -j%i' %
                                                cpu_count(output=conanfile.output)),
                         conanfile.command)

        cmake.generator = "NMake Makefiles"
        cmake.test()
        self.assertEqual('cmake --build '
                         '%s' % CMakeTest.scape('. --target test'),
                         conanfile.command)

    @pytest.mark.skipif(platform.system() != "Windows", reason="Only for Windows")
    def test_clean_sh_path(self):
        os.environ["PATH"] = os.environ.get("PATH", "") + os.pathsep + self.tempdir
        save(os.path.join(self.tempdir, "sh.exe"), "Fake sh")
        conanfile = ConanFileMock()
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "12"
        settings.arch = "x86"
        conanfile.settings = settings

        cmake = CMake(conanfile)
        cmake.configure()
        self.assertIn(self.tempdir, conanfile.path)

        cmake.generator = "MinGW Makefiles"
        cmake.configure()
        self.assertNotIn(self.tempdir, conanfile.path)

        # Automatic gcc
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "gcc"
        settings.compiler.version = "5.4"
        settings.arch = "x86"
        conanfile.settings = settings

        cmake = CMake(conanfile)
        cmake.configure()
        self.assertNotIn(self.tempdir, conanfile.path)

    def test_pkg_config_path(self):
        conanfile = ConanFileMock()
        conanfile.generators = ["pkg_config"]
        conanfile.folders.set_base_install("/my_install/folder/")
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "12"
        settings.arch = "x86"
        conanfile.settings = settings
        cmake = CMake(conanfile)
        cmake.configure()
        self.assertEqual(conanfile.captured_env["PKG_CONFIG_PATH"], "/my_install/folder/")

        conanfile.generators = []
        cmake = CMake(conanfile)
        cmake.configure()
        self.assertNotIn("PKG_CONFIG_PATH", conanfile.captured_env)

        cmake = CMake(conanfile)
        cmake.configure(pkg_config_paths=["reldir1", "/abspath2/to/other"])
        self.assertEqual(conanfile.captured_env["PKG_CONFIG_PATH"],
                         os.path.pathsep.join(["/my_install/folder/reldir1",
                                               "/abspath2/to/other"]))

        # If there is already a PKG_CONFIG_PATH do not set it
        conanfile.generators = ["pkg_config"]
        cmake = CMake(conanfile)
        with tools.environment_append({"PKG_CONFIG_PATH": "do_not_mess_with_this"}):
            cmake.configure()
            self.assertEqual(conanfile.captured_env["PKG_CONFIG_PATH"], "do_not_mess_with_this")

    def test_shared(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "12"
        settings.arch = "x86"
        settings.os = "Windows"

        conanfile = ConanFileMock(shared=True)
        conanfile.settings = settings
        cmake = CMake(conanfile)

        self.assertEqual(cmake.definitions["BUILD_SHARED_LIBS"], "ON")

        conanfile = ConanFileMock(shared=False)
        conanfile.settings = settings
        cmake = CMake(conanfile)

        self.assertEqual(cmake.definitions["BUILD_SHARED_LIBS"], "OFF")

        conanfile = ConanFileMock(shared=None)
        conanfile.settings = settings
        cmake = CMake(conanfile)

        self.assertNotIn("BUILD_SHARED_LIBS", cmake.definitions)

    def test_verbose(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "12"
        settings.arch = "x86"

        conanfile = ConanFileMock()
        conanfile.settings = settings
        cmake = CMake(conanfile)

        self.assertNotIn("CMAKE_VERBOSE_MAKEFILE", cmake.definitions)

        cmake.verbose = True
        self.assertEqual(cmake.definitions["CMAKE_VERBOSE_MAKEFILE"], "ON")

        cmake.verbose = False
        self.assertEqual(cmake.definitions["CMAKE_VERBOSE_MAKEFILE"], "OFF")

        cmake.definitions["CMAKE_VERBOSE_MAKEFILE"] = True
        self.assertTrue(cmake.verbose)

        cmake.definitions["CMAKE_VERBOSE_MAKEFILE"] = False
        self.assertFalse(cmake.verbose)

        del cmake.definitions["CMAKE_VERBOSE_MAKEFILE"]
        self.assertFalse(cmake.verbose)

    def test_set_toolset(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "15"
        settings.arch = "x86"
        settings.compiler.toolset = "v140"  # Will be overwritten by parameter

        conanfile = ConanFileMock()
        conanfile.settings = settings

        cmake = CMake(conanfile, toolset="v141")
        self.assertIn('-T "v141"', cmake.command_line)

        # DEPRECATED VARIABLE, NOT MODIFY ANYMORE THE TOOLSET
        with tools.environment_append({"CONAN_CMAKE_TOOLSET": "v141"}):
            cmake = CMake(conanfile)
            self.assertNotIn('-T "v141"', cmake.command_line)

        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "15"
        settings.arch = "x86"
        settings.compiler.toolset = "v140"

        conanfile = ConanFileMock()
        conanfile.settings = settings

        cmake = CMake(conanfile)
        self.assertIn('-T "v140"', cmake.command_line)

    @parameterized.expand([('Ninja',),
                           ('NMake Makefiles',),
                           ('NMake Makefiles JOM',)
                           ])
    def test_toolset_with_unsupported_generator(self, generator):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "15"
        settings.arch = "x86"
        settings.compiler.toolset = "v140"

        conanfile = ConanFileMock()
        conanfile.settings = settings

        with self.assertRaises(ConanException):
            cmake = CMake(conanfile, generator=generator)
            _ = cmake.command_line

    def test_missing_settings(self):
        def instance_with_os_build(os_build):
            settings = Settings.loads(get_default_settings_yml())
            settings.os_build = os_build
            conanfile = ConanFileMock()
            conanfile.settings = settings
            return CMake(conanfile)

        cmake = instance_with_os_build("Linux")
        self.assertEqual(cmake.generator, "Unix Makefiles")

        cmake = instance_with_os_build("Macos")
        self.assertEqual(cmake.generator, "Unix Makefiles")

        cmake = instance_with_os_build("Windows")
        self.assertEqual(cmake.generator, None)

        with tools.environment_append({"CONAN_CMAKE_GENERATOR": "MyCoolGenerator"}):
            cmake = instance_with_os_build("Windows")
            self.assertEqual(cmake.generator, "MyCoolGenerator")

    def test_cmake_system_version_android(self):
        with tools.environment_append({"CONAN_CMAKE_SYSTEM_NAME": "SomeSystem",
                                       "CONAN_CMAKE_GENERATOR": "SomeGenerator"}):
            settings = Settings.loads(get_default_settings_yml())
            settings.os = "WindowsStore"
            settings.os.version = "8.1"

            conanfile = ConanFileMock()
            conanfile.settings = settings
            cmake = CMake(conanfile)
            self.assertEqual(cmake.definitions["CMAKE_SYSTEM_VERSION"], "8.1")

            settings = Settings.loads(get_default_settings_yml())
            settings.os = "Android"
            settings.os.api_level = "32"

            conanfile = ConanFileMock()
            conanfile.settings = settings
            cmake = CMake(conanfile)
            self.assertEqual(cmake.definitions["CMAKE_SYSTEM_VERSION"], "32")

    def test_install_definitions(self):
        conanfile = ConanFileMock()
        conanfile.folders.set_base_package(None)
        conanfile.settings = Settings.loads(get_default_settings_yml())
        install_defintions = {"CMAKE_INSTALL_PREFIX": conanfile.package_folder,
                              "CMAKE_INSTALL_BINDIR": "bin",
                              "CMAKE_INSTALL_SBINDIR": "bin",
                              "CMAKE_INSTALL_LIBEXECDIR": "bin",
                              "CMAKE_INSTALL_LIBDIR": "lib",
                              "CMAKE_INSTALL_INCLUDEDIR": "include",
                              "CMAKE_INSTALL_OLDINCLUDEDIR": "include",
                              "CMAKE_INSTALL_DATAROOTDIR": "share"}

        # Without package_folder
        cmake = CMake(conanfile)
        for key, value in cmake.definitions.items():
            self.assertNotIn(key, install_defintions.keys())

        # With package_folder
        conanfile.folders.set_base_package("my_package_folder")
        install_defintions["CMAKE_INSTALL_PREFIX"] = conanfile.package_folder
        cmake = CMake(conanfile)
        for key, value in install_defintions.items():
            self.assertEqual(cmake.definitions[key], value)

    @parameterized.expand([("Macos", "10.9",),
                           ("iOS", "7.0",),
                           ("watchOS", "4.0",),
                           ("tvOS", "11.0",)])
    @mock.patch("platform.system", return_value="Darwin")
    @mock.patch("conans.client.tools.apple.XCRun.sdk_path", return_value='/opt')
    def test_cmake_system_version_osx(self, the_os, os_version, _, __):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = the_os

        # No version defined
        conanfile = ConanFileMock()
        conanfile.settings = settings
        cmake = CMake(conanfile)
        self.assertFalse("CMAKE_OSX_DEPLOYMENT_TARGET" in cmake.definitions)
        if the_os == "Macos":
            self.assertFalse("CMAKE_SYSTEM_NAME" in cmake.definitions)
        else:
            self.assertTrue("CMAKE_SYSTEM_NAME" in cmake.definitions)
        self.assertFalse("CMAKE_SYSTEM_VERSION" in cmake.definitions)

        # Version defined using Conan env variable
        with tools.environment_append({"CONAN_CMAKE_SYSTEM_VERSION": "23"}):
            conanfile = ConanFileMock()
            conanfile.settings = settings
            cmake = CMake(conanfile)
            self.assertEqual(cmake.definitions["CMAKE_SYSTEM_VERSION"], "23")
            self.assertEqual(cmake.definitions["CMAKE_OSX_DEPLOYMENT_TARGET"], "23")

        # Version defined in settings
        settings.os.version = os_version
        conanfile = ConanFileMock()
        conanfile.settings = settings
        cmake = CMake(conanfile)
        self.assertEqual(cmake.definitions["CMAKE_SYSTEM_VERSION"], os_version)
        self.assertEqual(cmake.definitions["CMAKE_OSX_DEPLOYMENT_TARGET"], os_version)

        # Version defined in settings AND using Conan env variable
        with tools.environment_append({"CONAN_CMAKE_SYSTEM_VERSION": "23"}):
            conanfile = ConanFileMock()
            conanfile.settings = settings
            cmake = CMake(conanfile)
            self.assertEqual(cmake.definitions["CMAKE_SYSTEM_VERSION"], "23")
            self.assertEqual(cmake.definitions["CMAKE_OSX_DEPLOYMENT_TARGET"], "23")

    @staticmethod
    def scape(args):
        pattern = "%s" if sys.platform == "win32" else r"'%s'"
        return ' '.join(pattern % i for i in args.split())

    @parameterized.expand([('Ninja', 'Visual Studio', 15),
                           ('NMake Makefiles', 'Visual Studio', 15),
                           ('NMake Makefiles JOM', 'Visual Studio', 15),
                           ('Ninja', 'clang', 6.0),
                           ('NMake Makefiles', 'clang', 6.0),
                           ('NMake Makefiles JOM', 'clang', 6.0)
                           ])
    @pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows vcvars")
    def test_vcvars_applied(self, generator, compiler, version):
        conanfile = ConanFileMock()
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = compiler
        settings.compiler.version = version
        conanfile.settings = settings

        cmake = CMake(conanfile, generator=generator)

        with mock.patch("conans.client.tools.vcvars_dict") as vcvars_mock:
            vcvars_mock.__enter__ = mock.MagicMock(return_value=(mock.MagicMock(), None))
            vcvars_mock.__exit__ = mock.MagicMock(return_value=None)
            cmake.configure()
            self.assertTrue(vcvars_mock.called, "vcvars weren't called")

        with mock.patch("conans.client.tools.vcvars_dict") as vcvars_mock:
            vcvars_mock.__enter__ = mock.MagicMock(return_value=(mock.MagicMock(), None))
            vcvars_mock.__exit__ = mock.MagicMock(return_value=None)
            cmake.build()
            self.assertTrue(vcvars_mock.called, "vcvars weren't called")

    @parameterized.expand([('Ninja',),
                           ('NMake Makefiles',),
                           ('NMake Makefiles JOM',),
                           ('Unix Makefiles',),
                           ])
    def test_intel_compilervars_applied(self, generator):
        conanfile = ConanFileMock()
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "intel"
        settings.arch = "x86_64"
        settings.compiler.version = "19"
        conanfile.settings = settings

        cmake = CMake(conanfile, generator=generator)

        with mock.patch("conans.client.tools.intel_compilervars_dict") as cvars_mock:
            cvars_mock.__enter__ = mock.MagicMock(return_value=(mock.MagicMock(), None))
            cvars_mock.__exit__ = mock.MagicMock(return_value=None)
            cmake.configure()
            self.assertTrue(cvars_mock.called, "intel_compilervars weren't called")

        with mock.patch("conans.client.tools.intel_compilervars_dict") as cvars_mock:
            cvars_mock.__enter__ = mock.MagicMock(return_value=(mock.MagicMock(), None))
            cvars_mock.__exit__ = mock.MagicMock(return_value=None)
            cmake.build()
            self.assertTrue(cvars_mock.called, "intel_compilervars weren't called")

    def test_cmake_program(self):
        conanfile = ConanFileMock()
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "14"
        conanfile.settings = settings

        cmake = CMake(conanfile, parallel=False)
        cmake.build()
        self.assertEqual("cmake --build %s" % CMakeTest.scape("."), conanfile.command)

        cmake = CMake(conanfile, cmake_program="use_another_cmake", parallel=False)
        cmake.build()
        self.assertEqual("use_another_cmake --build %s" % CMakeTest.scape("."), conanfile.command)

        with tools.environment_append({"CONAN_CMAKE_PROGRAM": "my_custom_cmake"}):
            cmake = CMake(conanfile, parallel=False)
            cmake.build()
            self.assertEqual("my_custom_cmake --build %s" % CMakeTest.scape("."), conanfile.command)

        with tools.environment_append({
            "CONAN_CMAKE_PROGRAM": "cmake_from_environment_has_priority"
        }):
            cmake = CMake(conanfile, cmake_program="but_not_cmake_from_the_ctor", parallel=False)
            cmake.build()
            self.assertEqual("cmake_from_environment_has_priority --build %s" % CMakeTest.scape("."),
                             conanfile.command)

    def test_msbuild_verbosity(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "10"
        settings.compiler.runtime = "MDd"
        settings.arch = "x86"
        settings.build_type = None

        conanfile = ConanFileMock()
        conanfile.settings = settings

        cmake = CMake(conanfile)
        cmake.build()
        self.assertIn("/verbosity:minimal", conanfile.command)

        cmake = CMake(conanfile, msbuild_verbosity="quiet")
        cmake.build()
        self.assertIn("/verbosity:quiet", conanfile.command)

        cmake = CMake(conanfile, msbuild_verbosity=None)
        cmake.build()
        self.assertNotIn("/verbosity", conanfile.command)

        with tools.environment_append({"CONAN_MSBUILD_VERBOSITY": "detailed"}):
            cmake = CMake(conanfile)
            cmake.build()
            self.assertIn("/verbosity:detailed", conanfile.command)

    def test_ctest_variables(self):
        conanfile = ConanFileMock()
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "14"
        conanfile.settings = settings

        cmake = CMake(conanfile, parallel=False, generator="NMake Makefiles")
        cmake.test()
        self.assertEqual(conanfile.captured_env["CTEST_OUTPUT_ON_FAILURE"], "0")
        self.assertNotIn("CTEST_PARALLEL_LEVEL", conanfile.captured_env)

        with tools.environment_append({"CONAN_CPU_COUNT": "666"}):
            cmake = CMake(conanfile, parallel=True, generator="NMake Makefiles")
            cmake.test(output_on_failure=True)
            self.assertEqual(conanfile.captured_env["CTEST_OUTPUT_ON_FAILURE"], "1")
            self.assertEqual(conanfile.captured_env["CTEST_PARALLEL_LEVEL"], "666")

    def test_unkown_generator_does_not_raise(self):
        # https://github.com/conan-io/conan/issues/4265
        settings = MockSettings({"os_build": "Windows", "compiler": "random",
                                 "compiler.version": "15", "build_type": "Release"})
        conanfile = ConanFileMock()
        conanfile.settings = settings
        cmake = CMake(conanfile)
        self.assertIsNone(cmake.generator)
        self.assertIn("WARN: CMake generator could not be deduced from settings", conanfile.output)
        cmake.configure()
        cmake.build()

    def test_cmake_system_version_windowsce(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "WindowsCE"
        settings.os.version = "8.0"

        conanfile = ConanFileMock()
        conanfile.settings = settings
        cmake = CMake(conanfile)
        self.assertEqual(cmake.definitions["CMAKE_SYSTEM_VERSION"], "8.0")

    def test_cmake_vs_arch(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.arch = "x86_64"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "15"

        conanfile = ConanFileMock()
        conanfile.settings = settings

        cmake = CMake(conanfile, generator="Visual Studio 15 2017 Win64", toolset="v141,host=x64")
        self.assertIn('-G "Visual Studio 15 2017 Win64"', cmake.command_line)
        self.assertIn('-T "v141,host=x64"', cmake.command_line)

        cmake = CMake(conanfile, generator="Visual Studio 15 2017", generator_platform="x64",
                      toolset="v141,host=x64")
        self.assertIn('-G "Visual Studio 15 2017 Win64"', cmake.command_line)
        self.assertIn('-T "v141,host=x64"', cmake.command_line)

    def test_skip_test(self):
        conf = ConfDefinition()
        conf.loads("tools.build:skip_test=1")
        conanfile = ConanFileMock()
        conanfile.settings = Settings()
        conanfile.conf = conf.get_conanfile_conf(None)
        cmake = CMake(conanfile, generator="Unix Makefiles")
        cmake.test()
        self.assertIsNone(conanfile.command)



import pytest

from conan.tools.gnu import AutotoolsToolchain
from conans.errors import ConanException
from conans.test.utils.mocks import ConanFileMock, MockSettings


def test_get_gnu_triplet_for_cross_building():
    """
    Testing AutotoolsToolchain and _get_gnu_triplet() function in case of
    having os=Windows and cross compiling
    """
    # Issue: https://github.com/conan-io/conan/issues/10139
    settings = MockSettings({"build_type": "Release",
                             "compiler": "gcc",
                             "compiler.version": "10.2",
                             "os": "Windows",
                             "arch": "x86_64"})
    conanfile = ConanFileMock()
    conanfile.settings = settings
    conanfile.settings_build = MockSettings({"os": "Solaris", "arch": "x86"})
    autotoolschain = AutotoolsToolchain(conanfile)
    assert autotoolschain._host == "x86_64-w64-mingw32"
    assert autotoolschain._build == "i686-solaris"


@pytest.mark.parametrize("runtime, runtime_type, expected",
                         [("static", "Debug", "MTd"),
                          ("static", "Release", "MT"),
                          ("dynamic", "Debug", "MDd"),
                          ("dynamic", "Release", "MD")])
def test_msvc_runtime(runtime, runtime_type, expected):
    """
    Testing AutotoolsToolchain with the msvc compiler adjust the runtime
    """
    # Issue: https://github.com/conan-io/conan/issues/10139
    settings = MockSettings({"build_type": "Release",
                             "compiler": "msvc",
                             "compiler.runtime": runtime,
                             "compiler.runtime_type": runtime_type,
                             "os": "Windows",
                             "arch": "x86_64"})
    conanfile = ConanFileMock()
    conanfile.settings = settings
    conanfile.settings_build = settings
    autotoolschain = AutotoolsToolchain(conanfile)
    expected_flag = "-{}".format(expected)
    assert autotoolschain.msvc_runtime_flag == expected_flag
    env = autotoolschain.environment().vars(conanfile)
    assert expected_flag in env["CFLAGS"]
    assert expected_flag in env["CXXFLAGS"]


@pytest.mark.parametrize("runtime", ["MTd", "MT", "MDd", "MD"])
def test_visual_runtime(runtime):
    """
    Testing AutotoolsToolchain with the msvc compiler adjust the runtime
    """
    # Issue: https://github.com/conan-io/conan/issues/10139
    settings = MockSettings({"build_type": "Release",
                             "compiler": "Visual Studio",
                             "compiler.runtime": runtime,
                             "os": "Windows",
                             "arch": "x86_64"})
    conanfile = ConanFileMock()
    conanfile.settings = settings
    conanfile.settings_build = settings
    autotoolschain = AutotoolsToolchain(conanfile)
    expected_flag = "-{}".format(runtime)
    assert autotoolschain.msvc_runtime_flag == expected_flag
    env = autotoolschain.environment().vars(conanfile)
    assert expected_flag in env["CFLAGS"]
    assert expected_flag in env["CXXFLAGS"]


def test_get_gnu_triplet_for_cross_building_raise_error():
    """
    Testing AutotoolsToolchain and _get_gnu_triplet() function raises an error in case of
    having os=Windows, cross compiling and not defined any compiler
    """
    # Issue: https://github.com/conan-io/conan/issues/10139
    settings = MockSettings({"build_type": "Release",
                             "os": "Windows",
                             "arch": "x86_64"})
    conanfile = ConanFileMock()
    conanfile.settings = settings
    conanfile.settings_build = MockSettings({"os": "Solaris", "arch": "x86"})
    with pytest.raises(ConanException) as conan_error:
        AutotoolsToolchain(conanfile)
        msg = "'compiler' parameter for 'get_gnu_triplet()' is not specified and " \
              "needed for os=Windows"
        assert msg == str(conan_error.value)



import platform
from unittest.mock import MagicMock, patch

import mock
import pytest
from mock.mock import PropertyMock

from conan.tools.system.package_manager import Apt, Dnf, Yum, Brew, Pkg, PkgUtil, Chocolatey, Zypper, \
    PacMan, _SystemPackageManagerTool
from conans import Settings
from conans.errors import ConanException
from conans.model.conf import Conf
from conans.test.utils.mocks import ConanFileMock


@pytest.mark.parametrize("platform, tool", [
    ("Linux", "apt-get"),
    ("Windows", "choco"),
    ("Darwin", "brew"),
    ("Solaris", "pkgutil"),
])
@pytest.mark.skipif(platform.system() != "Linux", reason="Only linux")
def test_package_manager_platform(platform, tool):
    with mock.patch("platform.system", return_value=platform):
        with mock.patch("distro.id", return_value=''):
            with mock.patch('conans.ConanFile.context', new_callable=PropertyMock) as context_mock:
                context_mock.return_value = "host"
                conanfile = ConanFileMock()
                conanfile.settings = Settings()
                manager = _SystemPackageManagerTool(conanfile)
                assert tool == manager.get_default_tool()


@pytest.mark.skipif(platform.system() != "Windows", reason="Only Windows")
def test_msys2():
    with mock.patch("platform.system", return_value="Windows"):
        with mock.patch('conans.ConanFile.context', new_callable=PropertyMock) as context_mock:
            context_mock.return_value = "host"
            conanfile = ConanFileMock()
            conanfile.conf = Conf()
            conanfile.settings = Settings()
            conanfile.conf["tools.microsoft.bash:subsystem"] = "msys2"
            manager = _SystemPackageManagerTool(conanfile)
            assert manager.get_default_tool() == "pacman"


@pytest.mark.parametrize("distro, tool", [
    ("ubuntu", "apt-get"),
    ("debian", "apt-get"),
    ("pidora", "yum"),
    ("rocky", "yum"),
    ("fedora", "dnf"),
    ("arch", "pacman"),
    ("opensuse", "zypper"),
    ("sles", "zypper"),
    ("opensuse", "zypper"),
    ("opensuse-tumbleweed", "zypper"),
    ("opensuse-leap", "zypper"),
    ("opensuse-next_version", "zypper"),
    ("freebsd", "pkg"),
])
@pytest.mark.skipif(platform.system() != "Linux", reason="Only linux")
def test_package_manager_distro(distro, tool):
    with mock.patch("platform.system", return_value="Linux"):
        with mock.patch("distro.id", return_value=distro):
            with mock.patch('conans.ConanFile.context', new_callable=PropertyMock) as context_mock:
                context_mock.return_value = "host"
                conanfile = ConanFileMock()
                conanfile.settings = Settings()
                manager = _SystemPackageManagerTool(conanfile)
                assert tool == manager.get_default_tool()


@pytest.mark.parametrize("sudo, sudo_askpass, expected_str", [
    (True, True, "sudo -A "),
    (True, False, "sudo "),
    (False, True, ""),
    (False, False, ""),
])
def test_sudo_str(sudo, sudo_askpass, expected_str):
    conanfile = ConanFileMock()
    conanfile.conf = Conf()
    conanfile.settings = Settings()
    conanfile.conf["tools.system.package_manager:sudo"] = sudo
    conanfile.conf["tools.system.package_manager:sudo_askpass"] = sudo_askpass
    with mock.patch('conans.ConanFile.context', new_callable=PropertyMock) as context_mock:
        context_mock.return_value = "host"
        apt = Apt(conanfile)
    assert apt.sudo_str == expected_str


@pytest.mark.parametrize("recommends, recommends_str", [
    (False, "--no-install-recommends "),
    (True, ""),
])
def test_apt_install_recommends(recommends, recommends_str):
    conanfile = ConanFileMock()
    conanfile.conf = Conf()
    conanfile.settings = Settings()
    conanfile.conf["tools.system.package_manager:tool"] = "apt-get"
    conanfile.conf["tools.system.package_manager:mode"] = "install"
    with mock.patch('conans.ConanFile.context', new_callable=PropertyMock) as context_mock:
        context_mock.return_value = "host"
        apt = Apt(conanfile)
        apt.install(["package1", "package2"], recommends=recommends)
    assert apt._conanfile.command == "apt-get install -y {}package1 package2".format(recommends_str)


@pytest.mark.parametrize("tool_class",
                         [Apt, Yum, Dnf, Brew, Pkg, PkgUtil, Chocolatey, PacMan, Zypper])
def test_tools_install_mode_check(tool_class):
    conanfile = ConanFileMock()
    conanfile.conf = Conf()
    conanfile.settings = Settings()
    conanfile.conf["tools.system.package_manager:tool"] = tool_class.tool_name
    with mock.patch('conans.ConanFile.context', new_callable=PropertyMock) as context_mock:
        context_mock.return_value = "host"
        tool = tool_class(conanfile)
        with pytest.raises(ConanException) as exc_info:
            def fake_check(*args, **kwargs):
                return ["package1", "package2"]
            from conan.tools.system.package_manager import _SystemPackageManagerTool
            with patch.object(_SystemPackageManagerTool, 'check', MagicMock(side_effect=fake_check)):
                tool.install(["package1", "package2"])
        assert exc_info.value.args[0] == "System requirements: 'package1, package2' are missing but " \
                                         "can't install because tools.system.package_manager:mode is " \
                                         "'check'.Please update packages manually or set " \
                                         "'tools.system.package_manager:mode' to 'install' in the [conf] " \
                                         "section of the profile, or in the command line using " \
                                         "'-c tools.system.package_manager:mode=install'"


@pytest.mark.parametrize("tool_class, result",
[
    (Apt, "apt-get update"),
    (Yum, "yum check-update -y"),
    (Dnf, "dnf check-update -y"),
    (Brew, "brew update"),
    (Pkg, "pkg update"),
    (PkgUtil, "pkgutil --catalog"),
    (Chocolatey, "choco outdated"),
    (PacMan, "pacman -Syyu --noconfirm"),
    (Zypper, "zypper --non-interactive ref"),
])
def test_tools_update_mode_install(tool_class, result):
    conanfile = ConanFileMock()
    conanfile.conf = Conf()
    conanfile.settings = Settings()
    conanfile.conf["tools.system.package_manager:tool"] = tool_class.tool_name
    for mode in ["check", "install"]:
        conanfile.conf["tools.system.package_manager:mode"] = mode
        with mock.patch('conans.ConanFile.context', new_callable=PropertyMock) as context_mock:
            context_mock.return_value = "host"
            tool = tool_class(conanfile)
            tool.update()
            if mode == "install":
                assert tool._conanfile.command == result
            else:
                # does not run the update when mode check
                assert tool._conanfile.command is None


@pytest.mark.parametrize("tool_class, result", [
    (Yum, "yum check-update -y"),
    (Dnf, "dnf check-update -y"),
])
def test_dnf_yum_return_code_100(tool_class, result):
    # https://github.com/conan-io/conan/issues/11661
    conanfile = ConanFileMock()
    conanfile.conf = Conf()
    conanfile.settings = Settings()
    conanfile.conf["tools.system.package_manager:tool"] = tool_class.tool_name
    conanfile.conf["tools.system.package_manager:mode"] = "install"
    with mock.patch('conans.ConanFile.context', new_callable=PropertyMock) as context_mock:
        context_mock.return_value = "host"
        tool = tool_class(conanfile)

        def fake_run(command, win_bash=False, subsystem=None, env=None, ignore_errors=False):
            assert command == result
            return 100 if "check-update" in command else 0

        conanfile.run = fake_run
        tool.update()

    #check that some random return code fails
    with mock.patch('conans.ConanFile.context', new_callable=PropertyMock) as context_mock:
        context_mock.return_value = "host"
        tool = tool_class(conanfile)

        def fake_run(command, win_bash=False, subsystem=None, env=None, ignore_errors=False):
            return 55 if "check-update" in command else 0

        conanfile.run = fake_run
        with pytest.raises(ConanException) as exc_info:
            tool.update()
        assert f"Command '{result}' failed" == str(exc_info.value)


@pytest.mark.parametrize("tool_class, result", [
    (Apt, 'apt-get install -y --no-install-recommends package1 package2'),
    (Yum, 'yum install -y package1 package2'),
    (Dnf, 'dnf install -y package1 package2'),
    (Brew, 'brew install package1 package2'),
    (Pkg, 'pkg install -y package1 package2'),
    (PkgUtil, 'pkgutil --install --yes package1 package2'),
    (Chocolatey, 'choco --install --yes package1 package2'),
    (PacMan, 'pacman -S --noconfirm package1 package2'),
    (Zypper, 'zypper --non-interactive in package1 package2'),
])
def test_tools_install_mode_install(tool_class, result):
    conanfile = ConanFileMock()
    conanfile.conf = Conf()
    conanfile.settings = Settings()
    conanfile.conf["tools.system.package_manager:tool"] = tool_class.tool_name
    conanfile.conf["tools.system.package_manager:mode"] = "install"
    with mock.patch('conans.ConanFile.context', new_callable=PropertyMock) as context_mock:
        context_mock.return_value = "host"
        tool = tool_class(conanfile)

        def fake_check(*args, **kwargs):
            return ["package1", "package2"]
        from conan.tools.system.package_manager import _SystemPackageManagerTool
        with patch.object(_SystemPackageManagerTool, 'check', MagicMock(side_effect=fake_check)):
            tool.install(["package1", "package2"])


@pytest.mark.parametrize("tool_class, result", [
    (Apt, 'dpkg-query -W -f=\'${Status}\' package | grep -q "ok installed"'),
    (Yum, 'rpm -q package'),
    (Dnf, 'rpm -q package'),
    (Brew, 'test -n "$(brew ls --versions package)"'),
    (Pkg, 'pkg info package'),
    (PkgUtil, 'test -n "`pkgutil --list package`"'),
    (Chocolatey, 'choco search --local-only --exact package | findstr /c:"1 packages installed."'),
    (PacMan, 'pacman -Qi package'),
    (Zypper, 'rpm -q package'),
])
def test_tools_check(tool_class, result):
    conanfile = ConanFileMock()
    conanfile.conf = Conf()
    conanfile.settings = Settings()
    conanfile.conf["tools.system.package_manager:tool"] = tool_class.tool_name
    with mock.patch('conans.ConanFile.context', new_callable=PropertyMock) as context_mock:
        context_mock.return_value = "host"
        tool = tool_class(conanfile)
        tool.check(["package"])



# coding=utf-8

import os
import textwrap

import pytest

from conans.model.ref import ConanFileReference
from conans.test.utils.scm import SVNLocalRepoTestCase
from conans.test.utils.tools import TestClient, load


@pytest.mark.tool_svn
class SVNTaggedComponentTest(SVNLocalRepoTestCase):
    # Reproducing https://github.com/conan-io/conan/issues/5017

    def setUp(self):
        # Create a sample SVN repository
        conanfile = textwrap.dedent("""
            from conans import ConanFile, tools

            class Lib(ConanFile):
                scm = {"type": "svn", "url": "auto", "revision": "auto"}
        """)
        files = {'trunk/level0.txt': "level0",
                 'trunk/level1/level1.txt': "level1",
                 'trunk/level1/conanfile.py': "invalid content",
                 'tags/sentinel': ""}
        self.project_url, _ = self.create_project(files=files)
        self.project_url = self.project_url.replace(" ", "%20")

        # Modify the recipe file and commit in trunk
        t = TestClient()
        t.run_command('svn co "{url}/trunk" "{path}"'.format(url=self.project_url,
                                                             path=t.current_folder))
        t.save({"level1/conanfile.py": conanfile})
        t.run_command('svn commit -m "created the conanfile"')

        # Create a tag for 'release 1.0'
        t.run_command('svn copy {url}/trunk {url}/tags/release-1.0'
                      ' -m "Release 1.0"'.format(url=self.project_url))

    def test_auto_tag(self):
        t = TestClient()
        ref = ConanFileReference.loads("lib/version@issue/testing")

        # Clone the tag to local folder
        url = os.path.join(self.project_url, "tags/release-1.0/level1").replace('\\', '/')
        t.run_command('svn co "{url}" "{path}"'.format(url=url, path=t.current_folder))

        # Export the recipe (be sure sources are retrieved from the repository)
        t.run("export . {ref}".format(ref=ref))
        package_layout = t.cache.package_layout(ref)
        exported_conanfile = load(package_layout.conanfile())
        self.assertNotIn("auto", exported_conanfile)
        self.assertIn('"revision": "3",', exported_conanfile)
        self.assertIn('tags/release-1.0/level1@3', exported_conanfile)
        t.run("remove {} -f -sf".format(ref))  # Remove sources caching

        # Compile (it will clone the repo)
        t.run("install {ref} --build=lib".format(ref=ref))
        self.assertIn("lib/version@issue/testing: SCM: Getting sources from url:", t.out)



import pytest

from conan.tools._check_build_profile import check_msg
from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient


@pytest.mark.parametrize("tool_class", ["CMakeDeps", "CMakeToolchain", "AutotoolsToolchain",
                                        "AutotoolsDeps", "MSBuildToolchain", "MSBuildDeps",
                                        "BazelDeps", "BazelToolchain", "MesonToolchain"])
def test_warning_message(tool_class):
    """FIXME: Remove in Conan 2.0"""
    client = TestClient()
    r = str(GenConanfile().with_name("lib").with_version("1.0")
            .with_settings("os", "arch", "build_type", "compiler")
            .with_import("from conan.tools.cmake import CMakeDeps, CMakeToolchain")
            .with_import("from conan.tools.gnu import AutotoolsDeps, AutotoolsToolchain")
            .with_import("from conan.tools.microsoft import MSBuildDeps, MSBuildToolchain")
            .with_import("from conan.tools.google import BazelDeps, BazelToolchain")
            .with_import("from conan.tools.meson import MesonToolchain"))

    r += """
    def generate(self):
        {}(self)
    """.format(tool_class)

    client.save({"conanfile.py": r})

    client.run("create . ")
    assert check_msg in client.out

    client.run("create . -pr:b=default")
    assert check_msg not in client.out


@pytest.mark.parametrize("cmake_deps_property", ["build_context_activated",
                                                 "build_context_build_modules",
                                                 "build_context_suffix",
                                                 False])
def test_error_cmake_deps_without_build_profile(cmake_deps_property):
    client = TestClient()
    r = str(GenConanfile().with_name("lib").with_version("1.0")
            .with_settings("os", "arch", "build_type", "compiler")
            .with_import("from conan.tools.cmake import CMakeDeps"))

    r += """
    def generate(self):
        deps = CMakeDeps(self)
        {}
        deps.generate()
    """.format("deps.{} = ['foo']".format(cmake_deps_property) if cmake_deps_property else "")

    client.save({"conanfile.py": r})

    client.run("create . ", assert_error=cmake_deps_property)
    if cmake_deps_property:
        assert "The 'build_context_activated' and 'build_context_build_modules' of the CMakeDeps " \
               "generator cannot be used without specifying a build profile. e.g: -pr:b=default"\
               in client.out

    client.run("create . -pr:b=default")
    assert "The 'build_context_activated' and 'build_context_build_modules' of the CMakeDeps " \
           "generator cannot be used without specifying a build profile. e.g: -pr:b=default" \
           not in client.out



import unittest

import six

from conans.model.build_info import DepsCppInfo, CppInfo, DepCppInfo


class DepsCppInfoTestCase(unittest.TestCase):

    def test_types(self):
        deps_cpp_info = DepsCppInfo()
        cpp_info = CppInfo("pkg", "rootpath")
        cpp_info.version = "version"
        cpp_info.libs = ["lib1", "lib2"]
        cpp_info.includedirs = ["include1"]
        deps_cpp_info.add("pkg", DepCppInfo(cpp_info))

        info_for_package = deps_cpp_info["pkg"]

        # Documented as list for 'self.cpp_info' object
        self.assertIsInstance(info_for_package.includedirs, list)
        self.assertIsInstance(info_for_package.libdirs, list)
        self.assertIsInstance(info_for_package.resdirs, list)
        self.assertIsInstance(info_for_package.bindirs, list)
        self.assertIsInstance(info_for_package.builddirs, list)
        self.assertIsInstance(info_for_package.libs, list)
        self.assertIsInstance(info_for_package.defines, list)
        self.assertIsInstance(info_for_package.cflags, list)
        self.assertIsInstance(info_for_package.cppflags, list)
        self.assertIsInstance(info_for_package.cxxflags, list)
        self.assertIsInstance(info_for_package.sharedlinkflags, list)
        self.assertIsInstance(info_for_package.exelinkflags, list)
        self.assertIsInstance(info_for_package.frameworks, list)
        self.assertIsInstance(info_for_package.frameworkdirs, list)
        self.assertIsInstance(info_for_package.rootpath, six.string_types)
        self.assertIsInstance(info_for_package.name, six.string_types)
        self.assertIsInstance(info_for_package.system_libs, list)
        self.assertIsInstance(info_for_package.build_modules, dict)
        self.assertIsInstance(info_for_package.components, dict)

        # Documented as list for `deps_cpp_info["pkg"]`
        self.assertIsInstance(info_for_package.include_paths, list)
        self.assertIsInstance(info_for_package.lib_paths, list)
        self.assertIsInstance(info_for_package.bin_paths, list)
        self.assertIsInstance(info_for_package.build_paths, list)
        self.assertIsInstance(info_for_package.res_paths, list)
        self.assertIsInstance(info_for_package.framework_paths, list)
        self.assertIsInstance(info_for_package.build_modules_paths, dict)
        self.assertIsInstance(info_for_package.get_name("generator"), six.string_types)
        self.assertIsInstance(info_for_package.version, six.string_types)
        self.assertIsInstance(info_for_package.components, dict)



import os
import platform
import textwrap
import unittest

import pytest

from conans.model.ref import ConanFileReference
from conans.test.utils.tools import TestClient
from conans.test.utils.tools import TestServer, TurboTestClient


@pytest.mark.skipif(platform.system() == "Windows", reason="Better to test only in NIX the symlinks")
class TestSkipBrokenSymlinks(unittest.TestCase):

    def test_package_broken_symlinks(self):
        server = TestServer()
        client = TurboTestClient(servers={"default": server})
        client2 = TurboTestClient(servers={"default": server})

        conanfile = """
import os
from conans import ConanFile, tools

class HelloConan(ConanFile):

    def package(self):
        # Link to file.txt and then remove it
        tools.save(os.path.join(self.package_folder, "file.txt"), "contents")
        os.symlink(os.path.join(self.package_folder, "file.txt"),
                   os.path.join(self.package_folder, "link.txt"))
        os.unlink(os.path.join(self.package_folder, "file.txt"))

"""
        ref = ConanFileReference.loads("lib/1.0@conan/stable")
        # By default it is not allowed
        client.create(ref, conanfile=conanfile, assert_error=True)
        self.assertIn("The file is a broken symlink", client.out)

        # Until we deactivate the checks
        client.run("config set general.skip_broken_symlinks_check=True")
        pref = client.create(ref, conanfile=conanfile)
        self.assertIn("Created package", client.out)
        p_folder = client.cache.package_layout(pref.ref).package(pref)

        # The link is there
        link_path = os.path.join(p_folder, "link.txt")
        self.assertTrue(os.path.islink(link_path))

        # The link is broken
        target_path = os.readlink(link_path)
        self.assertFalse(os.path.exists(target_path))

        # We can upload the package and reuse it
        client.upload_all(ref)

        client2.run("install {}".format(ref))
        self.assertIn("Downloaded package", client2.out)

    def test_broken_in_local_sources(self):
        conanfile = textwrap.dedent("""
            from conans import ConanFile, CMake

            class SymlinksConan(ConanFile):
                name = "symlinks"
                version = "1.0.0"
                exports_sources = "src/*"
            """)

        t = TestClient()
        t.save({'conanfile.py': conanfile, 'src/file': "content"})

        # Create a broken symlink
        broken_symlink = os.path.join(t.current_folder, 'src', 'link')
        os.symlink('not-existing', broken_symlink)

        # Check the bronken symlink locally
        self.assertTrue(os.path.islink(broken_symlink))
        self.assertFalse(os.path.exists(broken_symlink))
        self.assertFalse(os.path.exists(os.path.realpath(broken_symlink)))

        t.run("export . user/channel", assert_error=True)
        self.assertIn("ERROR: The file is a broken symlink", t.out)



import os
import textwrap
import unittest

from conans.model.ref import ConanFileReference, PackageReference
from conans.paths import CONANFILE, CONANFILE_TXT
from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import NO_SETTINGS_PACKAGE_ID, TestClient


class TestPackageInfo(unittest.TestCase):

    def test_package_info_called_in_local_cache(self):
        client = TestClient()
        conanfile_tmp = '''
from conans import ConanFile
import os

class HelloConan(ConanFile):
    name = "%s"
    version = "1.0"
    build_policy = "missing"
    options = {"switch": ["1",  "0"]}
    default_options = "switch=0"
    %s

    def build(self):
        self.output.warn("Env var MYVAR={0}.".format(os.getenv("MYVAR", "")))

    def package_info(self):
        if self.options.switch == "0":
            self.env_info.MYVAR = "foo"
        else:
            self.env_info.MYVAR = "bar"

'''
        for index in range(4):
            requires = "requires = 'Lib%s/1.0@conan/stable'" % index if index > 0 else ""
            conanfile = conanfile_tmp % ("Lib%s" % (index + 1), requires)
            client.save({CONANFILE: conanfile}, clean_first=True)
            client.run("create . conan/stable")

        txt = "[requires]\nLib4/1.0@conan/stable"
        client.save({CONANFILE_TXT: txt}, clean_first=True)
        client.run("install . -o *:switch=1")
        self.assertIn("Lib1/1.0@conan/stable: WARN: Env var MYVAR=.", client.out)
        self.assertIn("Lib2/1.0@conan/stable: WARN: Env var MYVAR=bar.", client.out)
        self.assertIn("Lib3/1.0@conan/stable: WARN: Env var MYVAR=bar.", client.out)
        self.assertIn("Lib4/1.0@conan/stable: WARN: Env var MYVAR=bar.", client.out)

        client.run("install . -o *:switch=0 --build Lib3")
        self.assertIn("Lib3/1.0@conan/stable: WARN: Env var MYVAR=foo", client.out)

    def test_package_info_name(self):
        dep = textwrap.dedent("""
            from conans import ConanFile

            class Dep(ConanFile):

                def package_info(self):
                    self.cpp_info.name = "MyCustomGreatName"
                """)
        intermediate = textwrap.dedent("""
            from conans import ConanFile

            class Intermediate(ConanFile):
                requires = "dep/1.0@us/ch"
                """)
        consumer = textwrap.dedent("""
            from conans import ConanFile

            class Consumer(ConanFile):
                requires = "intermediate/1.0@us/ch"

                def build(self):
                    for dep_key, dep_value in self.deps_cpp_info.dependencies:
                        self.output.info("%s name: %s" % (dep_key, dep_value.get_name('txt')))
                """)

        client = TestClient()
        client.save({"conanfile_dep.py": dep,
                     "conanfile_intermediate.py": intermediate,
                     "conanfile_consumer.py": consumer})
        client.run("create conanfile_dep.py dep/1.0@us/ch")
        client.run("create conanfile_intermediate.py intermediate/1.0@us/ch")
        client.run("create conanfile_consumer.py consumer/1.0@us/ch")
        self.assertIn("intermediate name: intermediate", client.out)
        self.assertIn("dep name: MyCustomGreatName", client.out)

    def test_package_info_system_libs(self):
        dep = textwrap.dedent("""
            from conans import ConanFile

            class Dep(ConanFile):

                def package_info(self):
                    self.cpp_info.system_libs = ["sysdep1"]
                """)
        intermediate = textwrap.dedent("""
            from conans import ConanFile

            class Intermediate(ConanFile):
                requires = "dep/1.0@us/ch"

                def package_info(self):
                    self.cpp_info.system_libs = ["sysdep2", "sysdep3"]
                """)
        consumer = textwrap.dedent("""
            from conans import ConanFile

            class Consumer(ConanFile):
                requires = "intermediate/1.0@us/ch"

                def build(self):
                    self.output.info("System deps: %s" % list(self.deps_cpp_info.system_libs))
                    for dep_key, dep_value in self.deps_cpp_info.dependencies:
                        self.output.info("%s system deps: %s" % (dep_key, list(dep_value.system_libs)))
                """)

        client = TestClient()
        client.save({"conanfile_dep.py": dep,
                     "conanfile_intermediate.py": intermediate,
                     "conanfile_consumer.py": consumer})
        client.run("create conanfile_dep.py dep/1.0@us/ch")
        client.run("create conanfile_intermediate.py intermediate/1.0@us/ch")
        client.run("create conanfile_consumer.py consumer/1.0@us/ch")
        dep_system_libs = ["sysdep1"]
        intermediate_system_libs = ["sysdep2", "sysdep3"]
        merged_system_libs = intermediate_system_libs + dep_system_libs
        self.assertIn("System deps: %s" % merged_system_libs, client.out)
        self.assertIn("intermediate system deps: %s" % intermediate_system_libs, client.out)
        self.assertIn("dep system deps: %s" % dep_system_libs, client.out)

    def test_package_info_components(self):
        dep = textwrap.dedent("""
            import os
            from conans import ConanFile

            class Dep(ConanFile):

                def package_info(self):
                    self.cpp_info.components["dep1"].libs.append("libdep1")
                    self.cpp_info.components["dep1"].defines.append("definedep1")
                    self.cpp_info.components["dep2"].libs.append("libdep2")
                    os.mkdir(os.path.join(self.package_folder, "include"))
                    os.mkdir(os.path.join(self.package_folder, "includedep2"))
                    self.cpp_info.components["dep2"].includedirs.append("includedep2")
                """)
        intermediate = textwrap.dedent("""
            import os
            from conans import ConanFile

            class Intermediate(ConanFile):
                requires = "dep/1.0@us/ch"

                def package_info(self):
                    self.cpp_info.components["int1"].requires = ["dep::dep"]  # To avoid exception
                    self.cpp_info.components["int1"].libs.append("libint1")
                    self.cpp_info.components["int1"].defines.append("defint1")
                    os.mkdir(os.path.join(self.package_folder, "include"))
                    os.mkdir(os.path.join(self.package_folder, "includeint1"))
                    self.cpp_info.components["int1"].includedirs.append("includeint1")
                    self.cpp_info.components["int2"].libs.append("libint2")
                    self.cpp_info.components["int2"].defines.append("defint2")
                    os.mkdir(os.path.join(self.package_folder, "includeint2"))
                    self.cpp_info.components["int2"].includedirs.append("includeint2")
                """)
        consumer = textwrap.dedent("""
            import os
            from conans import ConanFile

            class Consumer(ConanFile):
                requires = "intermediate/1.0@us/ch"

                def build(self):
                    self.output.info("deps_cpp_info.libs: %s" % list(self.deps_cpp_info.libs))
                    self.output.info("deps_cpp_info.defines: %s" % list(self.deps_cpp_info.defines))
                    self.output.info("deps_cpp_info.include_paths: %s" %
                        [os.path.basename(value) for value in self.deps_cpp_info.include_paths])
                    for dep_key, dep_value in self.deps_cpp_info.dependencies:
                        self.output.info("%s.libs: %s" % (dep_key, list(dep_value.libs)))
                        self.output.info("%s.defines: %s" % (dep_key, list(dep_value.defines)))
                        self.output.info("%s.include_paths: %s" % (dep_key,
                                         [os.path.basename(value) for value in
                                         dep_value.include_paths]))
                """)

        client = TestClient()
        client.save({"conanfile_dep.py": dep,
                     "conanfile_intermediate.py": intermediate,
                     "conanfile_consumer.py": consumer})
        client.run("export conanfile_dep.py dep/1.0@us/ch")
        client.run("export conanfile_intermediate.py intermediate/1.0@us/ch")
        client.run("create conanfile_consumer.py consumer/1.0@us/ch --build missing")

        self.assertIn("deps_cpp_info.libs: ['libint1', 'libint2', 'libdep1', 'libdep2']", client.out)
        self.assertIn("deps_cpp_info.defines: ['definedep1', 'defint1', 'defint2']", client.out)
        self.assertIn("deps_cpp_info.include_paths: ['include', 'includeint1', 'includeint2', "
                      "'include', 'includedep2']", client.out)

        self.assertIn("intermediate.libs: ['libint1', 'libint2']",
                      client.out)
        self.assertIn("intermediate.defines: ['defint1', 'defint2']",
                      client.out)
        self.assertIn("intermediate.include_paths: ['include', 'includeint1', 'includeint2']",
                      client.out)

        self.assertIn("dep.libs: ['libdep1', 'libdep2']", client.out)
        self.assertIn("dep.defines: ['definedep1']", client.out)
        self.assertIn("dep.include_paths: ['include', 'includedep2']", client.out)

    def test_package_info_raise_components(self):
        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class MyConan(ConanFile):

                def package_info(self):
                    self.cpp_info.defines.append("defint")
                    self.cpp_info.components["int1"].libs.append("libint1")
        """)
        client = TestClient()
        client.save({"conanfile.py": conanfile})
        client.run("create conanfile.py dep/1.0@us/ch", assert_error=True)
        self.assertIn("dep/1.0@us/ch package_info(): self.cpp_info.components cannot be used "
                      "with self.cpp_info global values at the same time", client.out)

        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class MyConan(ConanFile):

                def package_info(self):
                    self.cpp_info.release.defines.append("defint")
                    self.cpp_info.components["int1"].libs.append("libint1")
        """)
        client.save({"conanfile.py": conanfile})
        client.run("create conanfile.py dep/1.0@us/ch", assert_error=True)
        self.assertIn("dep/1.0@us/ch package_info(): self.cpp_info.components cannot be used "
                      "with self.cpp_info configs (release/debug/...) at the same time", client.out)

        conanfile = textwrap.dedent("""
                    from conans import ConanFile

                    class MyConan(ConanFile):

                        def package_info(self):
                            self.cpp_info.components["dep"].libs.append("libint1")
                """)
        client.save({"conanfile.py": conanfile})
        client.run("create conanfile.py dep/1.0@us/ch", assert_error=True)
        self.assertIn("dep/1.0@us/ch package_info(): Component name cannot be the same as the "
                      "package name: 'dep'", client.out)

    def test_package_info_components_complete(self):
        dep = textwrap.dedent("""
            import os
            from conans import ConanFile
            class Dep(ConanFile):
                exports_sources = "*"
                def package(self):
                    self.copy("*")
                def package_info(self):
                    self.cpp_info.name = "Galaxy"
                    self.cpp_info.components["Starlight"].includedirs = [os.path.join("galaxy", "starlight")]
                    self.cpp_info.components["Starlight"].libs = ["libstarlight"]
                    self.cpp_info.components["Planet"].includedirs = [os.path.join("galaxy", "planet")]
                    self.cpp_info.components["Planet"].libs = ["libplanet"]
                    self.cpp_info.components["Planet"].requires = ["Starlight"]
                    self.cpp_info.components["Launcher"].system_libs = ["ground"]
                    self.cpp_info.components["ISS"].includedirs = [os.path.join("galaxy", "iss")]
                    self.cpp_info.components["ISS"].libs = ["libiss"]
                    self.cpp_info.components["ISS"].libdirs = ["iss_libs"]
                    self.cpp_info.components["ISS"].system_libs = ["solar", "magnetism"]
                    self.cpp_info.components["ISS"].requires = ["Starlight", "Launcher"]
        """)
        consumer = textwrap.dedent("""
        from conans import ConanFile
        class Consumer(ConanFile):
            requires = "dep/1.0@us/ch"
            def build(self):
                # Global values
                self.output.info("GLOBAL Include paths: %s" % list(self.deps_cpp_info.include_paths))
                self.output.info("GLOBAL Library paths: %s" % list(self.deps_cpp_info.lib_paths))
                self.output.info("GLOBAL Binary paths: %s" % list(self.deps_cpp_info.bin_paths))
                self.output.info("GLOBAL Libs: %s" % list(self.deps_cpp_info.libs))
                self.output.info("GLOBAL System libs: %s" % list(self.deps_cpp_info.system_libs))
                # Deps values
                for dep_key, dep_value in self.deps_cpp_info.dependencies:
                    self.output.info("DEPS name: %s" % dep_value.get_name('txt'))
                    self.output.info("DEPS Include paths: %s" % list(dep_value.include_paths))
                    self.output.info("DEPS Library paths: %s" % list(dep_value.lib_paths))
                    self.output.info("DEPS Binary paths: %s" % list(dep_value.bin_paths))
                    self.output.info("DEPS Libs: %s" % list(dep_value.libs))
                    self.output.info("DEPS System libs: %s" % list(dep_value.system_libs))
                # Components values
                for dep_key, dep_value in self.deps_cpp_info.dependencies:
                    for comp_name, comp_value in dep_value.components.items():
                        self.output.info("COMP %s Include paths: %s" % (comp_name,
                            list(comp_value.include_paths)))
                        self.output.info("COMP %s Library paths: %s" % (comp_name, list(comp_value.lib_paths)))
                        self.output.info("COMP %s Binary paths: %s" % (comp_name, list(comp_value.bin_paths)))
                        self.output.info("COMP %s Libs: %s" % (comp_name, list(comp_value.libs)))
                        self.output.info("COMP %s Requires: %s" % (comp_name, list(comp_value.requires)))
                        self.output.info("COMP %s System libs: %s" % (comp_name, list(comp_value.system_libs)))
        """)

        client = TestClient()
        client.save({"conanfile_dep.py": dep, "conanfile_consumer.py": consumer,
                     "galaxy/starlight/starlight.h": "",
                     "lib/libstarlight": "",
                     "galaxy/planet/planet.h": "",
                     "lib/libplanet": "",
                     "galaxy/iss/iss.h": "",
                     "iss_libs/libiss": "",
                     "bin/exelauncher": ""})
        dep_ref = ConanFileReference("dep", "1.0", "us", "ch")
        dep_pref = PackageReference(dep_ref, NO_SETTINGS_PACKAGE_ID)
        client.run("create conanfile_dep.py dep/1.0@us/ch")
        client.run("create conanfile_consumer.py consumer/1.0@us/ch")
        package_folder = client.cache.package_layout(dep_ref).package(dep_pref)

        expected_comp_starlight_include_paths = [os.path.join(package_folder, "galaxy", "starlight")]
        expected_comp_planet_include_paths = [os.path.join(package_folder, "galaxy", "planet")]
        expected_comp_launcher_include_paths = []
        expected_comp_iss_include_paths = [os.path.join(package_folder, "galaxy", "iss")]
        expected_comp_starlight_library_paths = [os.path.join(package_folder, "lib")]
        expected_comp_launcher_library_paths = [os.path.join(package_folder, "lib")]
        expected_comp_planet_library_paths = [os.path.join(package_folder, "lib")]
        expected_comp_iss_library_paths = [os.path.join(package_folder, "iss_libs")]
        expected_comp_starlight_binary_paths = [os.path.join(package_folder, "bin")]
        expected_comp_launcher_binary_paths = [os.path.join(package_folder, "bin")]
        expected_comp_planet_binary_paths = [os.path.join(package_folder, "bin")]
        expected_comp_iss_binary_paths = [os.path.join(package_folder, "bin")]

        expected_global_include_paths = expected_comp_planet_include_paths + \
            expected_comp_iss_include_paths + expected_comp_starlight_include_paths
        expected_global_library_paths = expected_comp_starlight_library_paths + \
            expected_comp_iss_library_paths
        expected_global_binary_paths = expected_comp_starlight_binary_paths

        self.assertIn("GLOBAL Include paths: %s" % expected_global_include_paths, client.out)
        self.assertIn("GLOBAL Library paths: %s" % expected_global_library_paths, client.out)
        self.assertIn("GLOBAL Binary paths: %s" % expected_global_binary_paths, client.out)
        self.assertIn("GLOBAL Libs: ['libplanet', 'libiss', 'libstarlight']", client.out)
        self.assertIn("GLOBAL System libs: ['solar', 'magnetism', 'ground']", client.out)

        self.assertIn("DEPS name: Galaxy", client.out)
        self.assertIn("DEPS Include paths: %s" % expected_global_include_paths, client.out)
        self.assertIn("DEPS Library paths: %s" % expected_global_library_paths, client.out)
        self.assertIn("DEPS Binary paths: %s" % expected_global_binary_paths, client.out)
        self.assertIn("DEPS Libs: ['libplanet', 'libiss', 'libstarlight']", client.out)
        self.assertIn("DEPS System libs: ['solar', 'magnetism', 'ground']", client.out)

        self.assertIn("COMP Starlight Include paths: %s" % list(expected_comp_starlight_include_paths),
                      client.out)
        self.assertIn("COMP Planet Include paths: %s" % list(expected_comp_planet_include_paths,),
                      client.out)
        self.assertIn("COMP Launcher Include paths: %s" % list(expected_comp_launcher_include_paths),
                      client.out)
        self.assertIn("COMP ISS Include paths: %s" % list(expected_comp_iss_include_paths), client.out)
        self.assertIn("COMP Starlight Library paths: %s" % list(expected_comp_starlight_library_paths),
                      client.out)
        self.assertIn("COMP Planet Library paths: %s" % list(expected_comp_planet_library_paths),
                      client.out)
        self.assertIn("COMP Launcher Library paths: %s" % list(expected_comp_launcher_library_paths),
                      client.out)
        self.assertIn("COMP ISS Library paths: %s" % list(expected_comp_iss_library_paths), client.out)
        self.assertIn("COMP Starlight Binary paths: %s" % list(expected_comp_iss_binary_paths), client.out)
        self.assertIn("COMP Planet Binary paths: %s" % list(expected_comp_planet_binary_paths), client.out)
        self.assertIn("COMP Launcher Binary paths: %s" % list(expected_comp_launcher_binary_paths),
                      client.out)
        self.assertIn("COMP ISS Binary paths: %s" % list(expected_comp_iss_binary_paths), client.out)
        self.assertIn("COMP Starlight Libs: ['libstarlight']", client.out)
        self.assertIn("COMP Planet Libs: ['libplanet']", client.out)
        self.assertIn("COMP Launcher Libs: []", client.out)
        self.assertIn("COMP ISS Libs: ['libiss']", client.out)
        self.assertIn("COMP Starlight System libs: []", client.out)
        self.assertIn("COMP Planet System libs: []", client.out)
        self.assertIn("COMP Launcher System libs: ['ground']", client.out)
        self.assertIn("COMP ISS System libs: ['solar', 'magnetism']", client.out)
        self.assertIn("COMP Starlight Requires: []", client.out)
        self.assertIn("COMP Launcher Requires: []", client.out)
        self.assertIn("COMP Planet Requires: ['Starlight']", client.out)
        self.assertIn("COMP ISS Requires: ['Starlight', 'Launcher']", client.out)

    def test_package_requires_in_components_requires(self):
        client = TestClient()
        client.save({"conanfile1.py": GenConanfile("dep1", "0.1"),
                     "conanfile2.py": GenConanfile("dep2", "0.1")})
        client.run("create conanfile1.py")
        client.run("create conanfile2.py")

        conanfile = GenConanfile("consumer", "0.1") \
            .with_requirement("dep1/0.1") \
            .with_requirement("dep2/0.1") \
            .with_package_info(cpp_info={"components": {"kk": {"requires": []}}},
                               env_info={})
        client.save({"conanfile.py": conanfile})
        client.run("create conanfile.py", assert_error=True)
        self.assertIn("consumer/0.1 package_info(): Package require 'dep1' not used in "
                      "components requires", client.out)

        conanfile = GenConanfile("consumer", "0.1") \
            .with_requirement("dep1/0.1") \
            .with_requirement("dep2/0.1") \
            .with_package_info(cpp_info={"components": {"kk": {"requires": ["dep1::dep1"]}}},
                               env_info={})
        client.save({"conanfile.py": conanfile})
        client.run("create conanfile.py", assert_error=True)
        self.assertIn("consumer/0.1 package_info(): Package require 'dep2' not used in components "
                      "requires", client.out)

        conanfile = GenConanfile("consumer", "0.1") \
            .with_requirement("dep1/0.1") \
            .with_requirement("dep2/0.1") \
            .with_package_info(cpp_info={"components": {"kk": {"requires": ["dep1::dep1"]},
                                                        "kkk": {"requires": ["kk"]}}},
                               env_info={})
        client.save({"conanfile.py": conanfile})
        client.run("create conanfile.py", assert_error=True)
        self.assertIn("consumer/0.1 package_info(): Package require 'dep2' not used in components "
                      "requires", client.out)

        conanfile = GenConanfile("consumer", "0.1") \
            .with_requirement("dep1/0.1") \
            .with_requirement("dep2/0.1") \
            .with_package_info(cpp_info={"components": {"kk": {"requires": []},
                                                        "kkk": {"requires": ["kk"]}}},
                               env_info={})
        client.save({"conanfile.py": conanfile})
        client.run("create conanfile.py", assert_error=True)
        self.assertIn("consumer/0.1 package_info(): Package require 'dep1' not used in components "
                      "requires", client.out)

        conanfile = GenConanfile("consumer", "0.1") \
            .with_requirement("dep1/0.1") \
            .with_requirement("dep2/0.1") \
            .with_package_info(cpp_info={"components": {"kk": {"requires": ["dep2::comp"]},
                                                        "kkk": {"requires": ["dep3::dep3"]}}},
                               env_info={})
        client.save({"conanfile.py": conanfile})
        client.run("create conanfile.py", assert_error=True)
        self.assertIn("consumer/0.1 package_info(): Package require 'dep1' not used in components "
                      "requires", client.out)

        conanfile = GenConanfile("consumer", "0.1") \
            .with_requirement("dep2/0.1") \
            .with_package_info(cpp_info={"components": {"kk": {"requires": ["dep2::comp"]},
                                                        "kkk": {"requires": ["dep3::dep3"]}}},
                               env_info={})
        client.save({"conanfile.py": conanfile})
        client.run("create conanfile.py", assert_error=True)
        self.assertIn("consumer/0.1 package_info(): Package require 'dep3' declared in components "
                      "requires but not defined as a recipe requirement", client.out)

        conanfile = GenConanfile("consumer", "0.1") \
            .with_package_info(cpp_info={"components": {"kk": {"requires": ["dep2::comp"]}}},
                               env_info={})
        client.save({"conanfile.py": conanfile})
        client.run("create conanfile.py", assert_error=True)
        self.assertIn("consumer/0.1 package_info(): Package require 'dep2' declared in components "
                      "requires but not defined as a recipe requirement", client.out)

        conanfile = GenConanfile("consumer", "0.1") \
            .with_requirement("dep1/0.1") \
            .with_requirement("dep2/0.1") \
            .with_package_info(cpp_info={"components": {"kk": {"requires": ["dep2::comp"]},
                                                        "kkk": {"requires": ["dep1::dep1"]}}},
                               env_info={})
        client.save({"conanfile.py": conanfile})
        client.run("create conanfile.py")  # Correct usage

    def test_get_name_local_flow(self):
        # https://github.com/conan-io/conan/issues/7854
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Package(ConanFile):
                def package_info(self):
                    self.cpp_info.names["cmake_find_package"] = "GTest"
                    self.cpp_info.filenames["cmake_find_package"] = "GtesT"
                """)
        client.save({"conanfile.py": conanfile})
        client.run("create . gtest/1.0@")

        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Package(ConanFile):
                requires = 'gtest/1.0'

                def build(self):
                    info = self.deps_cpp_info['gtest'].get_name('cmake_find_package')
                    self.output.info("GTEST_INFO: %s" % info)
                    fileinfo = self.deps_cpp_info['gtest'].get_filename('cmake_find_package')
                    self.output.info("GTEST_FILEINFO: %s" % fileinfo)
            """)
        client.save({"conanfile.py": conanfile})
        client.run("create . pkg/1.0@")
        self.assertIn("pkg/1.0: GTEST_INFO: GTest", client.out)
        self.assertIn("pkg/1.0: GTEST_FILEINFO: GtesT", client.out)
        client.run("install . pkg/1.0@")
        client.run("build .")
        self.assertIn("conanfile.py (pkg/1.0): GTEST_INFO: GTest", client.out)
        self.assertIn("conanfile.py (pkg/1.0): GTEST_FILEINFO: GtesT", client.out)

    def test_none_folders(self):
        # https://github.com/conan-io/conan/issues/11856
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class pkgConan(ConanFile):
                def package_info(self):
                    self.cpp_info.resdirs = [None]
            """)
        client.save({"conanfile.py": conanfile})
        client.run("create . pkg/1.0@")
        assert "Created package revision" in client.out
